//---------------------------------------------------------------------------
// Burn - Arcade emulator library
// Copyright (c) 2001 Dave (www.finalburn.com), all rights reserved.

// This refers to all the code except where stated otherwise
// (e.g. ym2151 emulator)

// You can use, modify and redistribute this code freely as long as you
// don't do so commercially. This copyright notice must remain with the code.
// If your program uses this code, you must either distribute or link to the
// source code. If you modify or improve this code, you must distribute
// the source code improvements.

// Dave
// Homepage: www.finalburn.com
// E-mail:  dave@finalburn.com
//---------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

extern unsigned int nCurrentFrame;

inline static int GetCurrentFrame() {
	return nCurrentFrame;
}

extern int nBurnMask;

extern int NetPlay; // Treble Winner - Are we in NetPlay?

struct BurnRomInfo { int nLen; unsigned int nCrc; int nType; };
struct BurnInputInfo { char *szName; unsigned char nType; unsigned char *pVal;  char *szInfo;};
struct BurnArea { void *Data; int nLen; char *szName; };

extern int nBurnVer; // Version number of the library

// Cheat.cpp
extern unsigned char bCheatsAllowed; // addded by kev.
extern unsigned char nCheatCount;
extern unsigned int nApplyCheatEveryXFrame; // by gangta
extern struct CheatInfo* pCheatInfo;
extern unsigned char nCheatFrameCounter;
extern unsigned char nType1BeforeInitializedCount;

// dipsw.cpp
extern unsigned char nDefaultDIPSWValue[3];
extern unsigned char *nDIPSW[3];
extern struct DIPSWInfo* pDIPSWInfo;
extern unsigned char nDIPSWCount;

// burn.cpp
extern int (__cdecl *dprintf) (char *szFormat,...);
extern unsigned int nBurnDrvCount; // Count of game drivers
extern unsigned int nBurnDrvSelect; // Which game driver is selected

extern unsigned char *pBurnDraw; // Pointer to correctly sized bitmap
extern unsigned char *pSShotBuffer; // Pointer to DtosMem (for screenshot purposes)
extern int nBurnPitch; // Pitch between each line
extern int nBurnBpp;   // Bytes per pixel
extern int nBurnSoundChoice; // &1 play sound effects &2=play music
extern unsigned char nBurnLayer; // Can be used externally to select which layers to show
extern unsigned char nSpriteEnable; // Can be used externally to select which Sprites to show

int BurnLibInit();
int BurnLibExit();
extern int (*BurnExtInput) (unsigned int i); // App-defined func to get the state of each input
char *BurnDrvText(unsigned int i);
int BurnDrvGetZipName(char **pszName,unsigned int i);
int BurnDrvGetRomInfo(struct BurnRomInfo *pri,unsigned int i);
int BurnDrvGetRomName(char **pszName,unsigned int i,int nAka);
int BurnDrvGetInputInfo(struct BurnInputInfo *pii,unsigned int i);
int BurnDrvGetScreen(int *pnWidth,int *pnHeight);
int BurnDrvGetAspect(int *pnXAspect,int *pnYAspect);
int BurnDrvGetHardwareCode();
bool BurnDrvIsWorking();
int BurnDrvInit();
int BurnDrvExit();
int BurnDrvFrame();
int BurnDrvRedraw();
int BurnRecalcPal();
extern unsigned int (*BurnHighCol) (int r,int g,int b,int i);
int BurnAreaScan(int nAction,int *pnMin); // &1=for reading &2=for writing &4=Volatile &8=Non-Volatile
extern int (*BurnAcb) (struct BurnArea *pba); // Area callback

// load.cpp
// Application-defined rom loading function:
extern int (*BurnExtLoadRom)(unsigned char *Dest,int *pnWrote,int i);

// sound.cpp
extern int nBurnSoundRate;   // sample rate of sound
extern int nBurnSoundLen;    // length in samples per frame
extern short *pBurnSoundOut; // pointer to output buffer

#ifdef __cplusplus
} // End of extern "C"
#endif

#define HARDWARE_PREFIX_MISC 0x0000
#define HARDWARE_PREFIX_CAPCOM  0x0100
#define HARDWARE_PREFIX_SEGA    0x0200
#define HARDWARE_PREFIX_KONAMI  0x0300
#define HARDWARE_PREFIX_TOAPLAN 0x0400

#define NO_OF_HARDWARE_CODES 16

#define HARDWARE_MISC_MISC (HARDWARE_PREFIX_MISC | 0x00)

#define HARDWARE_CAPCOM_CPS1 (HARDWARE_PREFIX_CAPCOM | 0x01)
#define HARDWARE_CAPCOM_CPS1_QSOUND (HARDWARE_PREFIX_CAPCOM | 0x02)
#define HARDWARE_CAPCOM_CPS1_GENERIC (HARDWARE_PREFIX_CAPCOM | 0x03)
#define HARDWARE_CAPCOM_CPSCHANGER (HARDWARE_PREFIX_CAPCOM | 0x04)
#define HARDWARE_CAPCOM_CPS2 (HARDWARE_PREFIX_CAPCOM | 0x05)

#define HARDWARE_SEGA_AFTERBURNER (HARDWARE_PREFIX_SEGA | 0x01)
#define HARDWARE_SEGA_GALAXYFORCE (HARDWARE_PREFIX_SEGA | 0x02)
#define HARDWARE_SEGA_SYSTEM16A (HARDWARE_PREFIX_SEGA | 0x03)
#define HARDWARE_SEGA_SYSTEM16B (HARDWARE_PREFIX_SEGA | 0x04)
#define HARDWARE_SEGA_SYSTEM18 (HARDWARE_PREFIX_SEGA | 0x05)
#define HARDWARE_SEGA_HANGON (HARDWARE_PREFIX_SEGA | 0x06)
#define HARDWARE_SEGA_SPACEHARRIER (HARDWARE_PREFIX_SEGA | 0x07)
#define HARDWARE_SEGA_OUTRUN (HARDWARE_PREFIX_SEGA | 0x08)

#define HARDWARE_KONAMI_TMNT (HARDWARE_PREFIX_KONAMI | 0x01)

#define HARDWARE_TOAPLAN_RAIZING (HARDWARE_PREFIX_TOAPLAN | 0x01)
