// Burn - Arcade emulator library - internal code

// Standard headers
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "burn.h"

// Cheat engine additions
#include "cheat.h"

// DipSW engine additions
#include "dipsw.h"

// Make the INLINE macro
#undef INLINE
#define INLINE inline

// Compensate for Windows' need to be different from the rest of the world.
#ifdef _MSC_VER

 #undef strcasecmp
 #define strcasecmp _stricmp

 #undef strncasecmp
 #define strncasecmp _strnicmp

#endif

#ifdef __GNUC__

 #undef _strncmpi
 #define _strncmpi strncasecmp

 #undef _strnicmp
 #define _strnicmp strncasecmp

#endif

#ifdef _MSC_VER
 #pragma warning (disable:4514) //disable "unreferenced inline function removed" warning message
#endif

// Chip emulators ------------------------------------------------


extern "C" {
#include "driver.h"
#include "ym2151.h"
}

// sek.cpp
#include "sek.h"

// zet.cpp
#include "zet.h"

// --------------------------------------

struct BurnDriver {
	char *szText[7];
	// [0] = Short name for the driver
	// [1] = Full name of the game the driver is for
	// [2] = Notes
	// added by kev
	// [3] = Manufacturer
	// [4] = System
	// [5] = Date
	// [6] = Parent Rom (set to NULL if no parent)
	bool working;	// set to 1 for a working rom and 0 for non-working
	int players;	// Max number of players a game supports (so we can remove single player games from netplay)
	int hardware;	// Which type of hardware the game runs on
	// end
	int (*GetZipName)(char **pszName,unsigned int i);  // Function to get possible zip names
	int (*GetRomInfo)(struct BurnRomInfo *pri,unsigned int i);  // Function to get the length and crc of each rom
	int (*GetRomName)(char **pszName,unsigned int i,int nAka);  // Function to get the possible names for each rom
	int (*GetInputInfo)(struct BurnInputInfo *pii,unsigned int i); // Function to get the input info for the game
	int (*Init)(); int (*Exit)(); int (*Frame)(); int (*Redraw)(); int (*AreaScan)(int nAction,int *pnMin);
	unsigned char *pRecalcPal; int nWidth,nHeight; int nXAspect,nYAspect; // Screen width, height, x/y aspect
};

#define BurnDriverD BurnDriver		// Debug status
#define BurnDriverX BurnDriver		// Exclude from build

// burn.cpp
// Scan a small variable or structure
static INLINE void ScanVar(void *pv,int nSize,char *szName)
{
  struct BurnArea ba;
  memset(&ba,0,sizeof(ba));
  ba.Data=pv; ba.nLen=nSize; ba.szName=szName;
  BurnAcb(&ba);
}

#define SCAN_VAR(x) ScanVar(&x,sizeof(x),#x);

int BurnClearSize(int w,int h);
int BurnClearScreen();
static INLINE void PutPix(unsigned char *pPix,unsigned int c)
{
       if (nBurnBpp>=4) *((unsigned int *)pPix)=c;
  else if (nBurnBpp>=3)
  {
    pPix[0]=(unsigned char) c;
    pPix[1]=(unsigned char)(c>>8);
    pPix[2]=(unsigned char)(c>>16);
  }
  else if (nBurnBpp>=2) *((unsigned short *)pPix)=(unsigned short)c;
  else *pPix=(unsigned char)c;
}

int BurnByteswap(unsigned char *pm,int nLen);
extern int (*BurnScode) (unsigned int nCode);
int BurnScodePass(unsigned int n);

// Standard rom functions
#include "stdrom.h"

// load.cpp
int BurnLoadRom(unsigned char *Dest,int i,int nGap);
int BurnXorRom(unsigned char *Dest,int i,int nGap);
int BurnLoadBitField(unsigned char *pDest,unsigned char *pSrc,int nField,int nSrcLen);

