#include "cps.h"
// CPS - Memory
// Additions by Kev
unsigned int CpsMProt[4];
unsigned int CpsBID[3];
//end
static unsigned char *CpsMem=NULL,*CpsMemEnd=NULL;
unsigned char *CpsRam90=NULL;
unsigned char *CpsZRamC0=NULL,*CpsZRamF0=NULL;
unsigned char *CpsSavePal=NULL;
unsigned char *CpsSaveReg[MAX_RASTER + 1];
unsigned char *CpsSaveFrg[MAX_RASTER + 1];
static unsigned char *CpsSaveRegData = NULL;
static unsigned char *CpsSaveFrgData = NULL;
unsigned char *CpsRam708=NULL,*CpsReg=NULL,*CpsFrg=NULL;
unsigned char *CpsRamFF=NULL;

unsigned int nStateAllMin;
unsigned int nStateNVMin;

// This routine is called first to determine how much memory is needed
// and then to set up all the pointers.
static int CpsMemIndex()
{
  unsigned char *Next; Next=CpsMem;

    CpsRam90 =Next; Next+=0x030000; // Video Ram
    CpsRamFF =Next; Next+=0x010000; // Work Ram
    CpsReg   =Next; Next+=0x000100; // Registers

  if (Cps==2)
  {
    CpsRam708=Next; Next+=0x008000; // Obj Ram
    CpsZRamC0=Next; Next+=0x001000; // Z80 c000-cfff
    CpsZRamF0=Next; Next+=0x001000; // Z80 f000-ffff
    CpsFrg   =Next; Next+=0x000010; // 'Four' Registers (Registers at 0x400000)
  }

  if (Cps1Qs==1)
  {
    CpsZRamC0=Next; Next+=0x001000; // Z80 c000-cfff
    CpsZRamF0=Next; Next+=0x001000; // Z80 f000-ffff
  }

  CpsSaveRegData	= Next;	Next += 0x0100 * (MAX_RASTER + 1);	// Draw Copy of registers
  CpsSaveFrgData	= Next;	Next += 0x0010 * (MAX_RASTER + 1);	// Draw Copy of 'Four' Registers

  CpsSavePal		= Next;	Next += 0x001000;					// Draw Copy of Correct Palette

  // Sprite Masking init
  if(nBurnMask && Cps==2)
  {
	  ZBuf = (unsigned short*)Next; Next += 384 * 224 * 2;
  }

  CpsMemEnd  =Next;

  int i;
  for (i = 0; i < MAX_RASTER + 1; i++) {
  	CpsSaveReg[i] = CpsSaveRegData + i * 0x0100;
  	CpsSaveFrg[i] = CpsSaveFrgData + i * 0x0010;
  }

  return 0;
}

static int AllocateMemory()
{
  int nLen=0;
  // Find out how much memory is needed
  CpsMem=NULL; CpsMemIndex();
  nLen=CpsMemEnd-(unsigned char *)0;
  CpsMem=(unsigned char *)malloc(nLen); if (CpsMem==NULL) return 1;
  memset(CpsMem,0,nLen); // blank all memory
  CpsMemIndex(); // Index the allocated memory
  return 0;
}

int CpsMemInit()
{
  int nRet=0;
  nRet=AllocateMemory();  if (nRet!=0) return 1;

  SekOpen(0);
  // Map in memory:
  // 68000 Rom (as seen as is, through read)
  SekMemory(CpsRom,0,nCpsRomLen-1 ,SM_READ);

  // 68000 Rom (as seen decrypted, through fetch)
  if (nCpsCodeLen>0)
  {
    // Decoded part (up to nCpsCodeLen)
    SekMemory(CpsCode,0,nCpsCodeLen-1,SM_FETCH);
  }
  if (nCpsRomLen>nCpsCodeLen)
  {
    // The rest (up to nCpsRomLen)
    SekMemory(CpsRom+nCpsCodeLen,nCpsCodeLen,nCpsRomLen-1,SM_FETCH);
  }

  if (Cps==2) SekMemory(CpsRam708,0x708000,0x70ffff,SM_RAM); // separate Obj Ram
  SekMemory(CpsRam90 ,0x900000,0x92FFFF,SM_RAM); // Gfx Ram
  SekMemory(CpsRamFF ,0xFF0000,0xFFFFFF,SM_RAM); // Work Ram
  SekExt[0].ReadByte =CpsReadByte;
  SekExt[0].WriteByte=CpsWriteByte;
  SekExt[0].ReadWord =CpsReadWord;
  SekExt[0].WriteWord=CpsWriteWord;
  SekClose();

  return 0;
}

int CpsMemExit()
{
  // Deallocate all used memory
  if (CpsMem!=NULL) free(CpsMem);  CpsMem=NULL;
  return 0;
}

static int ScanRam()
{
  // scan ram:
  struct BurnArea ba;
  memset(&ba,0,sizeof(ba));

    ba.Data=CpsRam90;  ba.nLen=0x030000; ba.szName="CpsRam90";  BurnAcb(&ba);
    ba.Data=CpsRamFF;  ba.nLen=0x010000; ba.szName="CpsRamFF";  BurnAcb(&ba);
    ba.Data=CpsReg;    ba.nLen=0x000100; ba.szName="CpsReg";    BurnAcb(&ba);

  if (Cps==2)
  {
    ba.Data=CpsRam708; ba.nLen=0x008000; ba.szName="CpsRam708"; BurnAcb(&ba);
    ba.Data=CpsZRamC0; ba.nLen=0x001000; ba.szName="CpsZRamC0"; BurnAcb(&ba);
    ba.Data=CpsZRamF0; ba.nLen=0x001000; ba.szName="CpsZRamF0"; BurnAcb(&ba);
    ba.Data=CpsFrg;    ba.nLen=0x000010; ba.szName="CpsFrg";    BurnAcb(&ba);
  }

  if (Cps1Qs==1)
  {
    ba.Data=CpsZRamC0; ba.nLen=0x001000; ba.szName="CpsZRamC0"; BurnAcb(&ba);
    ba.Data=CpsZRamF0; ba.nLen=0x001000; ba.szName="CpsZRamF0"; BurnAcb(&ba);
  }

  return 0;
}

// Scan the current state of the CPS1/2 machine
int CpsAreaScan(int nAction,int *pnMin)
{
  if (CpsMem==NULL) return 1;

  if (nAction&8) // Scan non-volatile
  {
    if (Cps==2 || Cps1Qs==1 || PangEEP==1) CpsEepromScan(); // eeprom memory
//    if (pnMin!=NULL) *pnMin=0x008400; // Return minimum compatible version
    if (pnMin!=NULL) *pnMin=nStateAllMin; // Return minimum compatible version
  }

  if (nAction&4) // Scan volatile
  {
//    if (pnMin!=NULL) *pnMin=0x010002; // Return minimum compatible version
    if (pnMin!=NULL) *pnMin=nStateNVMin; // Return minimum compatible version
    ScanRam();
    if (Cps==1 && Cps1Qs==0) PsndScan(nAction&3); // PSound chips
    if (Cps==2 || Cps1Qs==1) QsndScan(nAction&3); // QSound chips
    SekScan(nAction&3); // scan 68000 state

    if (nAction&2) CpsRecalcPal=1; // Palette could have changed
  }

  return 0;
}


// CPS2 sound codes in Z80 memory:
int CpsScodePoll()
{
  int nCode=0;

  if (Cps!=2 && Cps1Qs==0) return 1;
  if (CpsZRamC0[0x00f]!=0x00) return 0;

  nCode =CpsZRamC0[0x000]<<8;
  nCode|=CpsZRamC0[0x001];

  // See if the code should be blocked
  if (BurnScodePass(nCode)==0) nCode=-1;

  // Assume nothing is being played:
  CpsZRamC0[0x00f]=0xff;

  if (nCode>=0)
  {
    CpsZRamC0[0x000]=(unsigned char)(nCode>>8);
    CpsZRamC0[0x001]=(unsigned char)(nCode&255);
    CpsZRamC0[0x00f]=0x00;
  }
  return 0;
}
