#include "cps.h"
// CPS - Read/Write

// Input bits
// Treble Winner (Increased number of ports from 8 to 11)
#define INP(nnn) unsigned char CpsInp##nnn[8];
CPSINPSET
#undef  INP

// Bytes to return from ports
#define INP(nnn) static unsigned char Inp##nnn;
CPSINPSET
#undef  INP

int PangEEP=0;
int Forgottn=0;
static int nCalc[2]={0,0};
//Hack for Forgottn Analogic Input
static unsigned char DialOne[3]={0x00, 0x14, 0xec};
static unsigned char DialTwo[3]={0x00, 0x00, 0x0f};

//CPS1 Slammasters protection
int slamPro=0;
unsigned char *CpsEncZRom=NULL;

#define INP(nnnn) unsigned char CpsInp##nnnn[8];
CPSINPEX
#undef  INP

#define INP(nnnn) static unsigned char Inp##nnnn;
CPSINPEX
#undef  INP


// Read input port 0x000-0x1ff
static INLINE unsigned char CpsReadPort(unsigned int ia)
{
  unsigned char d=0xff;

  if (ia==0x000) { d=(unsigned char)~Inp000; return d; }
  if (ia==0x001) { d=(unsigned char)~Inp001; return d; }
  if (ia==0x010) { d=(unsigned char)~Inp010; return d; }
  if (ia==0x011) { d=(unsigned char)~Inp011; return d; }

  if (ia==0x018) { d=(unsigned char)~Inp018; return d; }

  if (ia==0x01a) { d=(unsigned char)~Cpi01A; return d; }
  if (ia==0x01c) { d=(unsigned char)~Cpi01C; return d; }
  if (ia==0x01e) { d=(unsigned char)~Cpi01E; return d; }


  if (Cps==2)
  {
     // Used on CPS2 only I think
     if (ia==0x020) { d=(unsigned char)~Inp020; return d; }
     if (ia==0x021)
	 {
        d=(unsigned char)~Inp021;
        d&=0xfe; d |= CpsEepromRead(); // CPS2 EEPROM read
        return d;
	 }

     // CPS2 Volume control
     if (ia==0x030) { d=0xe0; return d; }
     if (ia==0x031) { d=0x21; return d; }
  }

  // Start of Board id additions
  if (Cps==1)
  {
	if(Forgottn)
	{
	  if (ia==0x053) return DialOne[Inp055&3];
	  if (ia==0x055) return DialTwo[Inp055&3];
      if (ia==0x05b) return DialOne[Inp05d&3];
	  if (ia==0x05d) return DialTwo[Inp05d&3];
	}

	if (ia==0x177) { d=(unsigned char)~Inp177; return d; }
	if (ia==0x179) { d=(unsigned char)~Inp179; return d; }

	// Treble Winner - sf2ue Input Ports
    if (ia==0x1fd) { d=(unsigned char)~Inp1fd; return d; }
    // end

	if (ia==0x100+CpsBID[0])     { d=(unsigned char)CpsBID[1]; return d; }
    if (ia==0x100+(CpsBID[0]+1)) { d=(unsigned char)CpsBID[2]; return d; }

  }

  if (Cps1Qs==1)
  {
     //CPS1 EEPROM read
     if (ia==0xc006 || ia==0xc007) { d = (unsigned char)CpsEepromRead(); return d; }
	 //CPS1 Player 3 (later games)
     if (ia==0xc000) { d=(unsigned char)~Inpc000; return d; }
     if (ia==0xc001) { d=(unsigned char)~Inpc001; return d; }
     //CPS1 Player 4
     if (ia==0xc002) { d=(unsigned char)~Inpc002; return d; }
     if (ia==0xc003) { d=(unsigned char)~Inpc003; return d; }
  }

  if (PangEEP==1)
  {
     //Pang3 EEPROM
     if (ia==0x17a || ia==0x17b) { d = (unsigned char)CpsEepromRead(); return d; }
  }

  return d;
}

// Write output port 0x000-0x1ff
static void CpsWritePort(unsigned int ia,unsigned char d)
{

  if (Cps==1 && Cps1Qs==0)
  {
    if (ia==0x181)
    {
	  // CPS1 sound code
      if (BurnScodePass(d)) { PsndCode=d; PsndIrq=1; }
	  return;
    }

	// CPS1 sound fade
    if (ia==0x189) { PsndFade=d; return; }
  }

  // CPS registers
  if (ia>=0x100 && ia<0x200)
  {
	  //Pang3 EEPROM
      if (PangEEP==1 && ia==0x17b && (d&0xc0)==0xc0) { Cps2EepromWrite(d&1); return; }
	  CpsReg[(ia^1)&0xff]=d; return;
  }

  if (Cps==2)
  {
	// CPS2 EEPROM write
    if (ia==0x040 && (d&0x60)==0x60) { Cps2EepromWrite((d>>4)&1); return; }
  }

  if (Cps1Qs==1)
  {
    //CPS1 EEPROM write
    if (ia==0xc007 && (d&0xc0)==0xc0) { Cps1QsEepromWrite(d&1); return; }
  }

}

unsigned char CpsReadByte(unsigned int a)
{
  unsigned char d=0;

  // Input ports mirrored between 0x800000 and 0x807fff
  if ((a&0xff8000)==0x800000) { d=CpsReadPort(a&0x1ff); return d;}

  if (Cps==2)
  {
    // QSound Z80
	if (a>=0x618000 && a<0x61a000) { d=CpsZRamC0[(a>>1)&0x0fff]; return d; }
  }

  if (Cps1Qs==1)
  {
    //CPS1 EEPROM & Player 3/4
    if (a>=0xf1c000 && a<=0xf1c007) { d=CpsReadPort(a&0xc00f); return d; }
    //CPS1 QSOUND
    if (a <=0xf19fff && a>=0xf18000) { d=CpsZRamC0[(a>>1)&0x0fff]; return d;}
    if (a <=0xf1e000 && a>=0xf1ffff) { d=CpsZRamF0[(a>>1)&0x0fff]; return d;}
    //CPS1 Slammasters protection
    if ((a <=0xf0ffff && a>=0xf00000) && slamPro==1) { d=CpsEncZRom[(a>>1)&0xffff]; return d;}
  }

  return d;
}

void CpsWriteByte(unsigned int a,unsigned char d)
{
  // Output ports mirrored between 0x800000 and 0x807fff
  if ((a&0xff8000)==0x800000) { CpsWritePort(a&0x1ff,d); return; }

  if (Cps==2)
  {
    // QSound Z80
    if (a>=0x618000 && a<0x61a000) { CpsZRamC0[(a>>1)&0x0fff]=d; return; }
    // 0x400000 registers
    if ((a&0xfffff0)==0x400000) CpsFrg[a&0x0f]=d;
  }

  if (Cps1Qs==1)
  {
    //CPS1 EEPROM
    if (a==0xf1c007) { CpsWritePort(a&0xc00f,d); return; }
    //CPS1 QSOUND
    if (a <=0xf19fff && a>=0xf18000) { CpsZRamC0[(a>>1)&0x0fff]=d; return;}
    if (a <=0xf1e000 && a>=0xf1ffff) { CpsZRamF0[(a>>1)&0x0fff]=d; return;}
  }

}

unsigned short CpsReadWord(unsigned int a)
{  
	if ((a&0xff8fff)==0x800100+CpsMProt[3])  
	{
	  return (unsigned short)((nCalc[0]*nCalc[1])>> 16);
	}
	// ports mirrored between 0x800000 and 0x807fff   
	if ((a&0xff8fff)==0x800100+CpsMProt[2])  
	{
	  return (unsigned short)((nCalc[0]*nCalc[1]));
	}
	
	SEK_DEF_READ_WORD(a)
}

void CpsWriteWord(unsigned int a,unsigned short d)
{  // ports mirrored between 0x800000 and 0x807fff
   if ((a&0xff8fff)==0x800100+CpsMProt[0])
		nCalc[0]=d;
   if ((a&0xff8fff)==0x800100+CpsMProt[1])
		nCalc[1]=d;
	
   SEK_DEF_WRITE_WORD(a,d)
}

// Reset all inputs to zero
static int InpBlank()
{
#define INP(nnn) Inp##nnn=0; memset(CpsInp##nnn,0,sizeof(CpsInp##nnn));
  CPSINPSET
#undef INP

#define INP(nnnn) Inp##nnnn=0; memset(CpsInp##nnnn,0,sizeof(CpsInp##nnnn));
  CPSINPEX
#undef INP

  return 0;
}

int CpsRwInit()
{
  InpBlank();
  return 0;
}

int CpsRwExit()
{
  InpBlank();
  return 0;
}

int CpsRwGetInp()
{
  // Compile separate buttons into Inpxxx
#define INP(nnn) \
  { int i=0; Inp##nnn=0; \
    for (i=0;i<8;i++) { Inp##nnn |= (CpsInp##nnn[i]&1)<<i; }  }
  CPSINPSET
#undef INP

 
#define INP(nnnn) \
  { int i=0; Inp##nnnn=0; \
    for (i=0;i<8;i++) { Inp##nnnn |= (CpsInp##nnnn[i]&1)<<i; }  }
  CPSINPEX
#undef INP

  return 0;
}
