// Definitions for compiling with GCC and Cygwin/MinGW
#ifdef __GNUC__
 #define _WIN32_IE 0x0300
 #ifndef STRICT
  #define STRICT
 #endif
#endif

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#ifdef _MSC_VER
 #pragma warning (push)
 #pragma warning (disable:4201)
#endif

#include <mmsystem.h>

#ifdef _MSC_VER
 #pragma warning (pop)
#endif

#define DIRECTINPUT_VERSION 0x0500 // Only need version from DirectX 5
#include <dinput.h>

// Quick fix for a deficiency in the Cygwin/MinGW win32 headers
#ifndef NMLVKEYDOWN
 #define NMLVKEYDOWN LV_KEYDOWN
#endif

// Compensate for Windows' need to be different from the rest of the world.
#ifdef _MSC_VER

 #undef strcasecmp
 #define strcasecmp _stricmp

 #undef strncasecmp
 #define strncasecmp _strnicmp

#endif

#ifdef __GNUC__

 #undef _strncmpi
 #define _strncmpi strncasecmp

 #undef _strnicmp
 #define _strnicmp strncasecmp

#endif

#include "burn.h"
#include "resource.h"

// Make the INLINE macro
#undef INLINE
#define INLINE inline

// Macro for releasing a COM object
#define RELEASE(x) { if ((x)!=NULL) (x)->Release();  (x)=NULL; }

#define APP_TITLE "Final Burn Alpha"

#define APP_TAKE_INPUT(hActive) ( hActive!=NULL && (hActive==hScrnWnd || hActive==hInpdDlg || hActive==hInpCheatDlg || hActive==hInpcX1Dlg || hActive==hInpcX2Dlg || hActive==hInpdDlg || hActive==hInpdX1Dlg || hActive==hInpdX2Dlg))

#define KEY_DOWN(Code) ((GetAsyncKeyState(Code)&0x8000)?1:0)
#define BZIP_MAX (4) // Maximum zip files to search through


#define MAX_GAMES 300

#define HANDLE_WM_ENTERMENULOOP(hwnd, wParam, lParam, fn) \
    ((fn)((hwnd), (BOOL)(wParam)), 0L)

#define HANDLE_WM_EXITMENULOOP(hwnd, wParam, lParam, fn) \
    ((fn)((hwnd), (BOOL)(wParam)), 0L)

#define HANDLE_MSGB(hwnd, message, fn)                       \
    case (message):                                          \
         HANDLE_##message((hwnd), (wParam), (lParam), (fn)); \
		 break;


#define HANDLE_MSGT(hwnd, message, fn)                       \
    case (message):                                          \
         HANDLE_##message((hwnd), (wParam), (lParam), (fn)); \
		 return TRUE;

// -----------------------------------------------

// main.cpp
extern HINSTANCE hAppInst; // Application Instance
extern HACCEL hAccel;
extern HINSTANCE hImm32;
extern int nAppVirtualFps; // virtual fps
extern int nUseSysMemory;
extern int bAppBassFilter;
extern int nMonAspectX;
extern int nMonAspectY;
extern int nAppShowCmd;
extern int nAppThreadPriority;
extern char szAppBurnVer[16];
extern bool bCmdOptUsed;

#define EXE_NAME_SIZE 32
extern char szAppExeName[EXE_NAME_SIZE + 1];

int AppError(char *szText,int bWarning);
int AppMessage(MSG *pMsg);

// media.cpp
extern HINSTANCE DDrawDll;
int MediaInit();
int MediaExit();

// dat.cpp
int write_datfile(int nDatType);
extern int nDatFileType;
// vid.cpp
// video output plugin:
struct VidOut
{
  int (*Init) ();
  int (*Exit) ();
  int (*Frame) (bool bRedraw = 0);
  int (*Paint) (int bValidate);
};
extern HWND hVidWnd; // The window to be used for video
extern int nVidWidth,nVidHeight,nVidDepth,nVidRefresh;
extern int nVidFullscreen;
extern int bVidScanlines;
extern int bVidScanHalf;
extern int bVidScanDelay;
extern int bVidCorrectAspect;
extern int bVidArcaderes;
int VidInit();
int VidExit();
int VidFrame();
int VidRedraw();
int VidPaint(int bValidate);


// dto.cpp
extern int bDtoStretch;
extern int bDtoTripleBuffer; // 1 to enable triple buffering

// misc.cpp
extern int bDoGamma;
extern float nGamma;

int AppDirectory();
int IntegerScale(RECT *pRect, int nGameWidth, int nGameHeight);
int GetClientScreenRect(HWND hWnd, RECT *pRect);
int SetBurnHighCol(int nDepth);
int WndInMid(HWND hMid, HWND hBase);
char *InputCodeDesc(int c);
void ComputeGammaLUT();

// drv.cpp
extern int bDrvOkay; // 1 if the Driver has been initted okay, and it's okay to use the BurnDrv functions
extern char szAppRomPath1[256];
extern char szAppRomPath2[256];
extern char szAppRomPath3[256];
extern char szAppRomPath4[256];
int DrvInit(int nDrvNum);
int DrvExit();

// run.cpp
extern int bRunPause;
extern int bReadInputEveryFrame;
extern int kNetGame;
extern int bAltPause;
int RunIdle();
int RunMessageLoop();
int RunReset();
void ToggleLayer(unsigned char thisLayer);

// zipfn.cpp
struct ZipEntry { char *szName; int nLen; unsigned int nCrc; };
int ZipOpen(char *szZip);
int ZipClose();
int ZipGetList(struct ZipEntry **pList,int *pnListCount);
int ZipLoadFile(unsigned char *Dest,int nLen,int *pnWrote,int nEntry);

#define ZIP_LIST_FREE(List,nListCount) \
{ if (List!=NULL) { int i; for (i=0;i<nListCount;i++) \
  { if (List[i].szName!=NULL) free(List[i].szName);  List[i].szName=NULL; } \
    free(List); } List=NULL; nListCount=0; }

// stringset.cpp
class StringSet
{
public:
  char *szText; int nLen;
  // printf function to add text to the Bzip string
  int __cdecl Add(char *szFormat,...);
  int Reset();
  StringSet();
  ~StringSet();
};

// bzip.cpp
extern char *szBzipName[BZIP_MAX*4+1];  // HyperYagami: New RomPaths
extern int nBzipError; // non-zero if there is a problem with the opened romset
extern StringSet BzipText; // Text which describes any problems with loading the zip
extern StringSet BzipDetail; // Text which describes in detail any problems with loading the zip
//ayeye
int BzipOpen(BOOL);
int BzipClose();
int BzipInit();
int BzipExit();

// DirectX -------------------------------------

// scrn.cpp
void CreateArcaderesItem();
void MenuLoadGame();
extern HWND hScrnWnd; // Handle to the screen window

//ayeye
extern int bRescanRoms;
int ScrnInit();
int ScrnExit();
int ScrnSize();
int ScrnTitle();

// dtos.cpp
int GetDisplayDepth();

// dinp.cpp
int DinpInit(HINSTANCE hInst,HWND hPassWnd);
int DinpExit();
int DinpStart(); // Call before checking for Input in a frame
int DinpState(int nCode);
int DinpFind();
int DinpAxis(int i,int nAxis);

// dsound.cpp
extern int nDSoundSamRate;           // sample rate
extern int nDSoundSegCount;          // Segs in the pdsbLoop buffer
extern int nDSoundSegLen;            // Seg length in samples (calculated from Rate/Fps)
extern int nDSoundFps;               // Application fps
extern short *DSoundNextSound;       // The next sound seg we will add to the sample loop
extern unsigned char bDSoundOkay;    // True if DSound was initted okay
extern unsigned char bDSoundPlaying; // True if the Loop buffer is playing
extern long nDSoundVol;
int DxSoundInit(HWND hWnd);
int DxSoundExit();
int DxSoundPlay();
int DxSoundStop();
int DxBlankSound();
int DxSoundCheck();
int DxSoundSetVolume();
extern int (*DSoundGetNextSound) (int bDraw); // Callback used to request more sound

// Aud.cpp
struct AudOut
{
	int (*BlankSound) ();
	int (*SoundCheck) ();
	int (*SoundInit) (HWND hWnd);
	int (*SoundPlay) ();
	int (*SoundStop) ();
	int (*SoundExit) ();
	int (*SoundSetVolume) ();
};
int AudSoundInit(HWND hWnd);
int AudSoundExit();
int AudSoundPlay();
int AudSoundStop();
int AudBlankSound();
int AudSoundCheck();
int AudSoundSetVolume();

// dsp.cpp
int DspInit();
int DspExit();
int DspDo(short *Wave,int nCount);
// -----------------------------------------------

int InputInit();
int InputExit();
int InputMake();
int InputTick();

// sel.cpp
//ayeye
extern int bLoadMenuAutoExpand;
extern int nLoadMenuShowX;
int SelDialog(); // Choose a Burn driver

// cona.cpp
struct VidPresetData {
	int nWidth;
	int nHeight;
};

extern struct VidPresetData VidPreset[4];

char *LabelCheck(char *s,char *szLabel);
int ConfigAppLoad();
int ConfigAppSave();

// cong.cpp
int ConfigGameLoad();
int ConfigGameSave();

#define QUOTE_MAX (128) // maximum length of "quoted strings"
int QuoteRead(char **pszQuote,char **pszEnd,char *szSrc); // Read a quoted string from szSrc and point to the end

// gami.cpp
#define SKIP_WS(s) for (;;) { if (*s!=' ' && *s!='\t') break;  s++; } // skip whitespace

struct GameInp
{
  unsigned char *pVal;  // Destination for the Input Value
  unsigned char nType;  // 0=binary (0,1) 1=analog (0x01-0xFF) 2=dip switch
                        // 3=cheat type 1 4=cheat type 2
                        // 5=dip-switch extended
  unsigned char nInput; // 0=undefined, 1=constant value, 2=on/off switch, 3=joystick axis, 4=slider
  unsigned char nConst; // the constant value
  unsigned short nCode; // the input code (for digital)
  unsigned char nJoy;   // the joystick number
  unsigned char nAxis;  // the joystick axis number
  unsigned short nSlider[2];  // Keys to use for slider
  short nSliderSpeed;   // speed which keys move the slider
  int nSliderValue;     // Current position of the slider
  short nSliderCenter;  // Speed the slider should center itself (high value = slow)
};
extern struct GameInp *GameInp;
extern unsigned int nGameInpCount;
int GameInpInit();
int GameInpExit();
char *InpToDesc(struct GameInp *pgi);
int GameInpBlank(int bDipSwitch);
int GameInpDefault();
int GameInpWrite(FILE *h);
int GameInpRead(char *szVal);

// Treble Winner: Player Default Controls
extern int nPlayer1, nPlayer2, nPlayer3, nPlayer4;

// panel.cpp
extern HWND hPanelDlg; // Handle to the Input Dialog
extern HMENU hMenu;
extern int bPanelDlgIsShown;
extern int nPanelHeight;
extern int bAutoPause;
int PanelShow(int bShow);
//ayeye
int PanelCreate(HWND hParent);
void MenuUpdate();
int MenuCreate(HWND hParent, int NetGame);

// wave.cpp
extern FILE *WaveLog; // wave log file
int WaveLogStart(char *szName,int nSamRate);
int WaveLogStop();

// text.cpp
int TextInit();
int TextExit();
int TextRender();
int TextCopy();

// gamc.cpp
int GamcMisc(struct GameInp *pgi,char *szi);
int GamcAnalogKey(struct GameInp *pgi,char *szi,int nPlayer,int nSlide);
int GamcAnalogJoy(struct GameInp *pgi,char *szi,int nPlayer,int nJoy,int nSlide);
int GamcPlayer(struct GameInp *pgi,char *szi,int nPlayer,int nDevice);
int GamcPlayerHotRod(struct GameInp *pgi,char *szi,int nPlayer,int nSide);

// inpd.cpp
extern HWND hInpdDlg; // Handle to the Input Dialog
int InpdUpdate();
int InpdCreate();
int InpdListMake(int bBuild);
//ayeye
int DefInpCreate();

// inpcheat.cpp
extern HWND hInpCheatDlg; // Handle to the Input Dialog
int InpCheatCreate();
int InpCheatListMake(int bBuild);

// inpdipsw.cpp
extern HWND hInpDIPSWDlg; // Handle to the Input Dialog
int InpDIPSWCreate();
int InpDIPSWListMake();

// inps.cpp
extern HWND hInpsDlg; // Handle to the Input Set Dialog
extern unsigned int nInpsInput; // The input number we are redefining
int InpsCreate();
int InpsUpdate();

// state.cpp
int StateLoad(char *szName,int bAll);
int StateSave(char *szName,int bAll);

// statec.cpp
int StateCompress(unsigned char **pDef,int *pnDefLen,int bAll);
int StateDecompress(unsigned char *Def,int nDefLen,int bAll);

// inpc.cpp
extern HWND hInpcDlg; // Handle to the Input Constant Dialog
extern unsigned int nInpcInput; // The input number we are redefining
int InpcCreate();


// inpcx1.cpp
extern HWND hInpcX1Dlg; // Handle to the Input Constant Dialog
extern unsigned int nInpcX1Input; // The input number we are redefining
int InpcX1Create();

// inpcx2.cpp
extern HWND hInpcX2Dlg; // Handle to the Input Constant Dialog
extern unsigned int nInpcX2Input; // The input number we are redefining
int InpcX2Create();

// inpdx1.cpp
extern HWND hInpdX1Dlg; // Handle to the Input Constant Dialog
extern struct DIPSWInfo *pInpdX1DIPSWInfo; // The DIPSW we are redefining
int InpdX1Create();

// inpdx2.cpp
extern HWND hInpdX2Dlg; // Handle to the Input Constant Dialog
extern struct DIPSWInfo *pInpdX2DIPSWInfo; // The DIPSW we are redefining
int InpdX2Create();

// stated.cpp
extern int bDrvSaveAll;
int StatedAuto(int bSave);
int StatedLoad(int nSlot = 0);
int StatedSave(int nSlot = 0);

//ayeye
// numdial.cpp
int NumDialCreate(int bDial);
int AboutCreate();
void GammaDialog();

//sfactd.cpp
int SFactdCreate();

// roms.cpp
extern BOOL gameAv[];
extern BOOL avOk;
extern int bAvbOnly;
int RomsDirCreate();
int TestRomsCreate();

//res.cpp
int ResCreate();

