// Burner Config file module
#include "app.h"

int nBurnMask = 1;

struct VidPresetData VidPreset[4] = {
	{ 400, 300 },
	{ 640, 480 },
	{ 1024, 768 },
	{ 1280, 960 },
};

static char szConfig[EXE_NAME_SIZE + 16];

static void CreateConfigName()
{
	memcpy(szConfig, "cfg\\", 4);
	memcpy(szConfig + 4, szAppExeName, strlen(szAppExeName));
	memcpy(szConfig + 4 + strlen(szAppExeName), ".ini", 5);

	return;
}

// Read in the config file for the whole application
int ConfigAppLoad()
{
  char szLine[256];
  FILE *h=NULL;

  CreateConfigName();

  h=fopen(szConfig,"rt");
  if (h==NULL) return 1;

  // Go through each line of the config file
  for (;;)
  {
    int nLen=0;
    if (fgets(szLine,sizeof(szLine),h)==NULL) break; // End of config file

    nLen=strlen(szLine);
    // Get rid of the linefeed at the end
    if (szLine[nLen-1]==10) { szLine[nLen-1]=0; nLen--; }

#define VAR(x) { char *szValue; szValue=LabelCheck(szLine,#x); \
  if (szValue!=NULL) x=strtol(szValue,NULL,0); }
#define FLT(x) { char *szValue; szValue=LabelCheck(szLine,#x); \
  if (szValue!=NULL) x=atof(szValue); }
#define STR(x) { char *szValue; szValue=LabelCheck(szLine,#x " "); \
  if (szValue!=NULL) strcpy(x,szValue); }

    // Video
    VAR(nUseSysMemory)
    VAR(nVidWidth) VAR(nVidHeight) VAR(nVidDepth) VAR(nVidRefresh)
    VAR(bVidArcaderes)
	VAR(VidPreset[0].nWidth) VAR(VidPreset[0].nHeight)
	VAR(VidPreset[1].nWidth) VAR(VidPreset[1].nHeight)
	VAR(VidPreset[2].nWidth) VAR(VidPreset[2].nHeight)
	VAR(VidPreset[3].nWidth) VAR(VidPreset[3].nHeight)
    VAR(bDoGamma)
    FLT(nGamma)
    VAR(bDtoStretch)
    VAR(bVidCorrectAspect)
    VAR(bDtoTripleBuffer)
    VAR(bVidScanlines)
    VAR(bVidScanHalf)
    VAR(bVidScanDelay)
    VAR(nMonAspectX)
    VAR(nMonAspectY)

    // Sound
    VAR(nDSoundSamRate)
    VAR(nDSoundSegCount)
    VAR(bAppBassFilter)
    VAR(nBurnSoundChoice)

    // Other
	// HyperYagami: New RomPaths
    STR(szAppRomPath1)
    STR(szAppRomPath2)
    STR(szAppRomPath3)
    STR(szAppRomPath4)

	VAR(bDrvSaveAll)
    VAR(bReadInputEveryFrame)
    VAR(nAppThreadPriority)

	//ayeye
    VAR(bAutoPause)
	VAR(nLoadMenuShowX)
	VAR(bLoadMenuAutoExpand)
	VAR(bAvbOnly)
	VAR(nBurnMask)

	// HyperYagami: # of cheats per second
	VAR(nApplyCheatEveryXFrame)

	// Treble Winner: Default Controls
	VAR(nPlayer1)
	VAR(nPlayer2)
	VAR(nPlayer3)
	VAR(nPlayer4)


#undef STR
#undef FLT
#undef VAR
  }

  fclose(h);
  return 0;
}

// Write out the config file for the whole application
int ConfigAppSave()
{
  FILE *h=NULL;

  if (bCmdOptUsed) {
	return 1;
  }

  CreateConfigName();

  h=fopen(szConfig,"wt");
  if (h==NULL) return 1;

  // Write title
  fprintf (h,"// " APP_TITLE " v%s Config File\n\n\n",szAppBurnVer);

#define VAR(x) fprintf (h,#x " %d\n",x);
#define FLT(x) fprintf (h,#x " %f\n",x);
#define STR(x) fprintf (h,#x " %s\n",x);

    fprintf (h,"// Video --------------------------------\n");

    fprintf (h,"\n// The display mode to use for fullscreen\n");
    VAR(nVidWidth) VAR(nVidHeight) VAR(nVidDepth)
    fprintf (h,"\n// Specify the refresh rate -- may not work with all video cards\n");
    VAR(nVidRefresh)
    fprintf (h,"\n// If non-zero, use the same resolution as the original arcade game\n");
    VAR(bVidArcaderes)
    fprintf (h,"\n// These are the preset resolutions appearing in the menu\n");
	VAR(VidPreset[0].nWidth) VAR(VidPreset[0].nHeight)
	VAR(VidPreset[1].nWidth) VAR(VidPreset[1].nHeight)
	VAR(VidPreset[2].nWidth) VAR(VidPreset[2].nHeight)
	VAR(VidPreset[3].nWidth) VAR(VidPreset[3].nHeight)
    fprintf (h,"\n// Perform gamma correction\n");
    VAR(bDoGamma)
    fprintf (h,"\n// Gamma to correct with\n");
    FLT(nGamma)
    fprintf (h,"\n// If non-zero, allow stretching of the image to any size\n");
    VAR(bDtoStretch)
    fprintf (h,"\n// If non-zero, stretch the image to the largest size preserving aspect ratio\n");
    VAR(bVidCorrectAspect)
    fprintf (h,"\n// If non-zero, try to use a triple buffer in fullscreen\n");
    VAR(bDtoTripleBuffer)
    fprintf (h,"\n// If non-zero, draw scanlines to simulate a low-res monitor\n");
    VAR(bVidScanlines)
    fprintf (h,"\n// If non-zero, draw odd scanlines inbetween too at 50%% intensity\n");
    VAR(bVidScanHalf)
    fprintf (h,"\n// If non-zero, delay odd scanlines by one frame (reduces flicker)\n");
    VAR(bVidScanDelay)
    fprintf (h,"\n// The aspect ratio of the monitor\n");
    VAR(nMonAspectX)
    VAR(nMonAspectY)
    fprintf (h,"\n// Buffering method: -1=Autodetect best memory, 0=force video memory, 1=force system memory\n");
    VAR(nUseSysMemory)
    fprintf (h,"\n\n\n");

    fprintf (h,"// Sound --------------------------------\n");
    fprintf (h,"// The sample rate\n");
    VAR(nDSoundSamRate)
    fprintf (h,"\n// Number of frames in sound buffer (= sound lag)\n");
    VAR(nDSoundSegCount)
    fprintf (h,"\n// If non-zero, filter the sound output to add more bass\n");
    VAR(bAppBassFilter)
    fprintf (h,"\n// Which sound codes to pass\n");
    VAR(nBurnSoundChoice)
    fprintf (h,"\n\n\n");

    fprintf (h,"// Other --------------------------------\n\n");
    fprintf (h,"// The paths to search for rom zips. (Should end with a backslash)\n");
    STR(szAppRomPath1)
    STR(szAppRomPath2)
    STR(szAppRomPath3)
    STR(szAppRomPath4)

    fprintf (h,"\n// If non-zero, load and save all ram (the state).\n");
    VAR(bDrvSaveAll)
    fprintf (h,"\n// If non-zero, read input for every frame even if the graphics is\n");
    fprintf (h,"// frame-skipping (controls could be more responsive, but emu could be slower).\n");
    fprintf (h,"// This option has no efect during netplay (Always 1 during netplay).\n");
    VAR(bReadInputEveryFrame)
    fprintf (h,"\n// The thread priority for the application. Do NOT edit this manually.\n");
    VAR(nAppThreadPriority)
	//ayeye
	fprintf (h,"\n// If non-zero, active auto pause.\n");
    VAR(bAutoPause)
	fprintf (h,"\n// Romsets to display.\n");
	VAR(nLoadMenuShowX)
	fprintf (h,"\n// Auto Expand All.\n");
	VAR(bLoadMenuAutoExpand)
    fprintf (h,"\n// Display only available.\n");
	VAR(bAvbOnly)
	fprintf (h,"\n// Enable sprite masking.\n");
	VAR(nBurnMask)

fprintf (h,"\n// Cheat frequency.  0=60 times/sec, 1=30 times/sec,...,X=(60/(1+X)) times/sec.\n");

	VAR(nApplyCheatEveryXFrame)


	// Treble Winner: Default Controls
    fprintf (h, "\n// Player default controls, 0=Keyboard, 1=Joystick 1, 2=Joystick 2, 3=Joystick 3\n");
   	fprintf (h, "// 4=HotRod 1, 5=HotRod 2\n");
    VAR(nPlayer1)
    VAR(nPlayer2)
    VAR(nPlayer3)
    VAR(nPlayer4)


	fprintf (h,"\n\n\n");

#undef STR
#undef FLT
#undef VAR

  fclose(h);
  return 0;
}

char *LabelCheck(char *s,char *szLabel)
{
  int nLen;
  if (s==NULL) return NULL;  if (szLabel==NULL) return NULL;
  nLen=strlen(szLabel);
  SKIP_WS(s) // skip whitespace

  if (strncmp(s,szLabel,nLen)!=0) return NULL; // doesn't match
  return s+nLen;
}

int QuoteRead(char **pszQuote,char **pszEnd,char *szSrc) // Read a quoted string from szSrc and point to the end
{
  static char szQuote[QUOTE_MAX]; char *s=NULL; int i=0;
  s=szSrc;
  SKIP_WS(s) // skip whitespace

  if (*s!='\"') return 1; // didn't start with a quote
  s++;

  // Copy the characters into szQuote
  for (i=0; i<QUOTE_MAX-1; i++,s++)
  {
    if (*s=='\"') { s++; break; } if (*s==0) break; // stop copying if end of line or end of string
    szQuote[i]=*s; // copy character to szQuote
  }
  szQuote[i]=0; // zero-terminate szQuote

  if (pszQuote!=NULL) *pszQuote=szQuote;
  if (pszEnd!=NULL)   *pszEnd=s;
  return 0;
}

