// Driver Init module
#include "app.h"

int bDrvOkay=0; // 1 if the Driver has been initted okay, and it's okay to use the BurnDrv functions

//HyperYagami: Initialize new RomPaths
char szAppRomPath1[256]="roms\\";
char szAppRomPath2[256]="roms2\\";
char szAppRomPath3[256]="roms3\\";
char szAppRomPath4[256]="roms4\\";

static int DoLibInit() // Do Init of Burn library driver
{
  int i=0, j=0; int nRet=0;
  //HyperYagami: we need 16 entries
  static char *szRomName[BZIP_MAX*4];

  //HyperYagami: Local RomPaths in array form,
  //it's easier to deal with (for-next loop)
  char *myRomPath[4];

  memset(&szRomName,0,sizeof(szRomName));

  //HyperYagami: Initial our local RomPaths
  myRomPath[0] = szAppRomPath1;
  myRomPath[1] = szAppRomPath2;
  myRomPath[2] = szAppRomPath3;
  myRomPath[3] = szAppRomPath4;

  for (i=0;i<BZIP_MAX;i++)
  {
	  //HyperYagami: more nStrLen
	char *szName=NULL; int nRet=0; int nStrLen[4]={0,0,0,0};

    nRet=BurnDrvGetZipName(&szName,i);
    if (nRet!=0) break;

	//HyperYagami: Initialize the stuff just like the way it was
	//But do that 4 times for each RomPath
	for (j=0;j<=3;j++)
	{
		// Allocate space for each full pathname
		nStrLen[j]=strlen(szName)+strlen(myRomPath[j]);
		szRomName[i*4+j]=(char *)malloc(nStrLen[j]+1); if (szRomName[i*4+j]==NULL) continue;
		sprintf (szRomName[i*4+j],"%s%s",myRomPath[j],szName); // Use path plus default zip file name
		// Point bzip to it
		szBzipName[i*4+j]=szRomName[i*4+j];
	}
  }
  szBzipName[i*4]=NULL; // Mark end of zip name list

  // Open the zip file(s) and check if they are okay
  BzipOpen(FALSE);
  // If there is an error with the romset, report it
  if (nBzipError!=0)
  {
    char *szTitle; int nIcon=0,nButton=0; int nRet=0;
    // Make the correct title and icon
    if (nBzipError&1) { nIcon=MB_ICONERROR;   szTitle=APP_TITLE " Error";   }
    else              { nIcon=MB_ICONWARNING; szTitle=APP_TITLE " Warning"; }

    if (nBzipError&0x08) nButton=MB_OK; // no data at all - pretty basic!
    else
    {
      BzipText.Add("\nWould you like more detailed information?\n(For experts only!)\n");
      nButton=MB_DEFBUTTON2|MB_YESNO;
    }

    nRet=MessageBox(hScrnWnd,BzipText.szText,  szTitle,nButton|nIcon);
    if (nRet==IDYES)
    {
      // Give the more detailed information string
      MessageBox(hScrnWnd,BzipDetail.szText,szTitle,MB_OK|nIcon);
    }
  }

  nRet=BurnDrvInit(); // Init the driver

  BzipClose();

  for (i=0;i<BZIP_MAX*4;i++)
  {
    // Free the names of each pathnames
    if (szRomName[i]!=NULL) free(szRomName[i]); // Free the strings
    szRomName[i]=NULL; szBzipName[i]=NULL; // Blank BZip list
  }
  szBzipName[i]=NULL; // Mark end of zip name list

  if (nRet!=0) return 1;

  return 0;
}

int DrvInit(int nDrvNum)
{
  int nRet=0;
  DrvExit(); // Make sure exitted
  
  nBurnDrvSelect=nDrvNum; // set the driver number

  nRet=DoLibInit(); // Init the Burn library's driver

  if (nRet!=0)
  {
    char szTemp[512];

    BurnDrvExit(); // Exit the driver

    ScrnTitle();
    sprintf (szTemp,"There was an error starting '%s'.\n",BurnDrvText(1));
    MessageBox(hScrnWnd,szTemp,APP_TITLE " Error",MB_OK|MB_ICONERROR);
    return 1;
  }

  GameInpInit(); // Init game input
  // Load config for the game
  nRet=ConfigGameLoad(); if (nRet!=0) ConfigGameSave(); // create initial game config file

  // Dip switch values in driver will have been set now:
  GameInpDefault(); // Auto-config any input which is still undefined to defaults
  // (also sets/gets dip switch valeus from driver)

  bDrvOkay=1; // Okay to use the BurnDrv functions

  extern int  kNetGame;            
  if(!kNetGame) StatedAuto(0);    

  ScrnTitle();
  ScrnSize();

  nBurnLayer=0xff; // show all layers

  // Init the video plugin
  nRet=VidInit();
  if (nRet!=0) { AppError("VidInit Failed",0); return 1; }

  // Reset the speed throttling code, so we don't 'jump' after the load
  RunReset();

  return 0;
}

int DrvExit()
{
  VidExit();

  InvalidateRect(hScrnWnd,NULL,1); UpdateWindow(hScrnWnd); // blank scrn window

  DestroyWindow(hInpdDlg); // Make sure the Input Dialog is exitted

  if (nBurnDrvSelect<nBurnDrvCount)
  {
    StatedAuto(1); // Save nv (or full) ram

    // Save config for the game
    if (bDrvOkay) ConfigGameSave(); // Only save config if we initted okay
    GameInpExit(); // Exit game input
    BurnDrvExit(); // Exit the driver
  }

  bDrvOkay=0; // Stop using the BurnDrv functions

  bRunPause=0; // Don't pause when exitted

  if (bDSoundOkay)
  {
    // Write silence into the sound buffer on exit, and for drivers which don't use pBurnSoundOut
    memset(DSoundNextSound,0,nDSoundSegLen<<2);
  }

  nBurnDrvSelect=~0U; // no driver selected
  return 0;
}
