// DirectDraw Three Output (dto) code
#include "app.h"
#include "dto.h"

IDirectDraw2 *DtoDD = NULL;				// DirectDraw interface
IDirectDrawSurface *DtoPrim = NULL;		// Primary surface
IDirectDrawSurface *DtoBack = NULL;		// Back buffer surface

int bDtoStretch = 0;
int bDtoTripleBuffer = 0;				// 1 to enable triple buffering
static int nDispStatus = 0;

typedef HRESULT (WINAPI *DDCreateFun) (GUID FAR *lpGUID, LPDIRECTDRAW FAR *lplpDD, IUnknown FAR *pUnkOuter );
typedef HRESULT (WINAPI *DDCreateClipperFun) ( DWORD dwFlags, LPDIRECTDRAWCLIPPER FAR *lplpDDClipper, IUnknown FAR *pUnkOuter );

static struct { int origX; int origY; int destX; int destY; } ConvertSize[] = {
	{ 224, 384, 400, 384 },
	{ 240, 320, 424, 320 }
};

// Get the appropriate screensize for vertical games
static bool GetVertScreenSize(int* width, int* height)
{
	for (int i = 0; i < sizeof(ConvertSize); i++) {
		if (ConvertSize[i].origX == *width && ConvertSize[i].origY == *height ) {
			*width = ConvertSize[i].destX;
			*height = ConvertSize[i].destY;
			return true;
		}
	}
	return false;
}

static bool SetArcaderes(int* nWidth, int* nHeight)
{
	int nGameWidth, nGameHeight;
	if (bDrvOkay) {
		BurnDrvGetScreen(&nGameWidth, &nGameHeight);
		if (nGameWidth < nGameHeight) {				// Vertical games
			if (GetVertScreenSize(&nGameWidth, &nGameHeight)) {
				*nWidth = nGameWidth;
				*nHeight = nGameHeight;
			} else {
				return false;
			}
		} else {									// Horizontal games
			*nWidth = nGameWidth;
			*nHeight = nGameHeight;
		}
	} else {
	 	return false;
	}
	return true;
}

int DtoClear(IDirectDrawSurface *Surf)
{
	// Clear a surface to black
	DDBLTFX BltFx;

	if (Surf == NULL) {
		return 1;
	}

	memset(&BltFx, 0, sizeof(BltFx));
	BltFx.dwSize = sizeof(BltFx);

	if (Surf->Blt(NULL,NULL,NULL,DDBLT_COLORFILL,&BltFx) < 1) {
		return 1;
	} else {
		return 0;
	}
}

int DtoPrimClear()
{
	if (DtoPrim == NULL) {
		return 1;
	}
	DtoClear(DtoPrim);					// Clear 1st page

	if (DtoBack == NULL) {
		return 0;
	}
	DtoClear(DtoBack);					// Clear 2nd page
	DtoPrim->Flip(NULL, DDFLIP_WAIT);	// wait till the flip actually occurs
	DtoClear(DtoBack);					// Clear 3rd page

	return 0;
}

static int DtoPrimInit(int bTriple)
{
	DDSURFACEDESC ddsd; int nRet=0;

	// Create the primary surface
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	if (bTriple) {
		// Make a primary surface capable of triple bufferring
		ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX | DDSCAPS_VIDEOMEMORY;
		ddsd.dwBackBufferCount = 2;
	} else {
		ddsd.dwFlags = DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	}

	DtoPrim = NULL;
	nRet = DtoDD->CreateSurface(&ddsd, &DtoPrim, NULL);
	if (nRet < 0 || DtoPrim == NULL) {
		return 1;
	}

	if (bTriple == 0) {						// We are done
		return 0;
	}

	// Get the back buffer
	memset(&ddsd.ddsCaps, 0, sizeof(ddsd.ddsCaps));
	ddsd.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;

	DtoBack = NULL;
	nRet = DtoPrim->GetAttachedSurface(&ddsd.ddsCaps, &DtoBack);
	if (nRet < 0 || DtoBack == NULL) {		// Failed to make triple buffer
		RELEASE(DtoPrim)
		return 1;
	}

	DtoPrimClear();							// Clear all three pages
	return 0;
}

static int DtoClipInit()
{
	IDirectDrawClipper *pddcClipper = NULL;		// The clipper
	DDCreateClipperFun DDCreateClipper = NULL;

	if (DDrawDll == NULL) {
		return 1;
	}

	DDCreateClipper = (DDCreateClipperFun)GetProcAddress(DDrawDll, "DirectDrawCreateClipper");
	if (DDCreateClipper) {						// Create a clipper
		DDCreateClipper(0, &pddcClipper,NULL);
	}

	if (pddcClipper) {
		if (pddcClipper->SetHWnd(0, hVidWnd) >= 0) {
			DtoPrim->SetClipper(pddcClipper);
			RELEASE(pddcClipper)
		}
	}
	return 0;
}

static int DtoExit();

static int DtoInit()
{
	DDCreateFun DDCreate = NULL;
	IDirectDraw *dd1 = NULL;
	int nWidth, nHeight;
	int nRet;

	// Dynamically get the DirectDrawCreate function
	if (DDrawDll == NULL) {
		return 1;
	}
	DDCreate = (DDCreateFun)GetProcAddress(DDrawDll, "DirectDrawCreate");
	if (DDCreate == NULL) {
		return 1;
	}

	// Create the DirectDraw object
	if (DDCreate(NULL, &dd1, NULL) < 0) {
		return 1;
	}
	if (dd1 == NULL) {
		return 1;
	}

	// Get the higher DirectDraw interface
	DtoDD = NULL;
	dd1->QueryInterface(IID_IDirectDraw2, (void **)&DtoDD);
	RELEASE(dd1)
	if (DtoDD == NULL) {
		return 1;
	}

	DtoPrim = NULL;							// No primary surface yet
	DtoBack = NULL;

	// Remember the changes to the display
	nDispStatus = 0;
	if (nVidFullscreen) {
		char szTemp[256] = "";
		if (DtoDD->SetCooperativeLevel(hVidWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN) >= 0) nDispStatus |= 1;	// changed coop level

		if (bVidArcaderes) {
			SetArcaderes(&nWidth, &nHeight);
		} else {
			nWidth = nVidWidth;
			nHeight = nVidHeight;
		}

		nRet = DtoDD->SetDisplayMode(nWidth, nHeight, nVidDepth, nVidRefresh, 0);
		if (nRet >= 0) nDispStatus |= 2;	// set display mode

		if (nRet < 0) {
			if (bVidArcaderes) {
				sprintf (szTemp,"Error setting \"%dx%dx%dbpp (%dhz)\" display mode\n\nThis is not a standard VGA resolution; please make sure it is supported by your video card.", nWidth, nHeight, nVidDepth, nVidRefresh);
			} else {
				sprintf (szTemp,"Error setting \"%dx%dx%dbpp (%dhz)\" display mode", nWidth, nHeight, nVidDepth, nVidRefresh);
			}
			AppError(szTemp, 1);
		}

		if (bDtoTripleBuffer) {
			if (DtoPrimInit(1)) {			// Try to make triple buffer
				AppError("Could not make Triple Buffer", 1);
				// If we fail, fail entirely and make a normal buffer
				RELEASE(DtoBack)
				RELEASE(DtoPrim)
			}
		}
	} else {
		DtoDD->SetCooperativeLevel(hVidWnd, DDSCL_NORMAL);
	}

	if (DtoPrim == NULL) {
		// No primary surface yet, so try normal
		if (DtoPrimInit(0)) {
			RELEASE(DtoBack)
			RELEASE(DtoPrim)
		}
	}

	if (DtoPrim == NULL) {					// No primary surface
		DtoExit();
		return 1;
	}

	// Make a clipper in windowed mode
	if (nVidFullscreen == 0) {
		DtoClipInit();
	}

	// Init the buffer surfaces
	DtosInit();

	return 0;
}

static int DtoExit()
{
	DtosExit();

	RELEASE(DtoPrim)					// a single call releases all surfaces
	DtoBack = NULL;

	// Undo the changes we made to the display
	if (nDispStatus & 2) {
		DtoDD->RestoreDisplayMode();
	}
	if (nDispStatus & 1) {
		DtoDD->SetCooperativeLevel(hVidWnd, DDSCL_NORMAL);
	}
	nDispStatus = 0;

	RELEASE(DtoDD)
	return 0;
}

// The video output plugin:
struct VidOut VidOutDto = {DtoInit, DtoExit, DtosFrame, DtosPaint};
