// DirectDraw Three Output Screen (dtos) code
#include "app.h"
#include "dto.h"
#include "chat.h"

static int nDtosWidth = 0, nDtosHeight = 0;		// screen size
static int nDtosDepth = 0, nDtosBpp = 0;
static int bDtosScan = 0;
static unsigned char *DtosMem = NULL;			// Memory buffer
static int nDtosMemPitch = 0;					// Memory buffer pitch
IDirectDrawSurface *pddsDtos = NULL;			// The screen surface
static int nHalfMask = 0;

// For the "OSD" functionality
static struct { char pMsgText[32]; COLORREF nColour; int nPriority; int nTimer; } DtosShortMsg = {"", 0, 0, 0,};
HFONT ShortMsgFont = NULL;

// Kill, Kill, Kill!!!
void DtosAddChat(const char *) {}

int GetDisplayDepth()
{
	return nDtosDepth;
}

inline static int GetSurfaceDepth(IDirectDrawSurface *pddsSurf)
{
	int nDepth = 8;
	DDPIXELFORMAT ddpf;
	if (pddsSurf == NULL) {
		return nDepth;
	}

	// Find out the pixelformat of the screen surface
	memset(&ddpf, 0, sizeof(ddpf));
	ddpf.dwSize = sizeof(ddpf);

	if (pddsSurf->GetPixelFormat(&ddpf) >= 0) {
		nDepth = ddpf.dwRGBBitCount;
		if (nDepth == 16 && ddpf.dwGBitMask == 0x03e0) nDepth = 15;
	}
	return nDepth;
}

// Create a buffer in normal memory
static int DtosMakeMem()
{
	int nMemLen = 0;
	nDtosMemPitch = nDtosWidth * nDtosBpp;
	nMemLen = nDtosHeight * nDtosMemPitch;
	DtosMem = (unsigned char *)malloc(nMemLen);

	if (DtosMem) {
		memset(DtosMem, 0, nMemLen);
		return 0;
	} else {
		return 1;
	}
}

static int AutodetectUseSys()
{
	// Try to autodetect the best secondary buffer type to use, based on the cards capabilities
	DDCAPS ddc;
	if (DtoDD == NULL) {
		return 1;
	}
	memset(&ddc, 0, sizeof(ddc));
	ddc.dwSize = sizeof(ddc);
	DtoDD->GetCaps(&ddc, NULL);

	if (ddc.dwCaps & DDCAPS_BLTSTRETCH) {	// If it can do a hardware stretch use video memory
		return 0;
	} else {								// Otherwise use system memory:
		return 1;
	}
}

// Create a secondary DD surface for the screen
static int DtosMakeSurf()
{
	int nRet;
	int nUseSys = 0;
	DDSURFACEDESC ddsd;

	if (DtoDD == NULL) {
		return 1;
	}

	nUseSys = nUseSysMemory;
	if (nUseSys < 0) {
		nUseSys = AutodetectUseSys();
	}

	// Try to allocate buffer in Video memory first, if that fails use System memory
	for (nRet = -1; nUseSys <= 1 && nRet < 0; nUseSys++) {
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
		if (nUseSys == 0) {
			ddsd.ddsCaps.dwCaps |= DDSCAPS_VIDEOMEMORY;
		} else {
			if (nUseSys == 1) {
				ddsd.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;
			}
		}

		ddsd.dwWidth = nDtosWidth;
		ddsd.dwHeight = nDtosHeight;
		if (bDtosScan) {
			ddsd.dwHeight <<= 1;
		}

		nRet = DtoDD->CreateSurface(&ddsd, &pddsDtos, NULL);
	}

	if (nRet < 0 || pddsDtos == NULL) {
		return 1;
	}

	DtoClear(pddsDtos);

	return 0;
}

void DtosShortMsgInit()
{
	if (ShortMsgFont) {
		DeleteObject(ShortMsgFont);
	}
	ShortMsgFont = CreateFont((18 << bDtosScan), 9, 0, 0, FW_DEMIBOLD - (bDtosScan * 100), 0, 0, 0, 0, 0, 0, 0, ANTIALIASED_QUALITY, "Lucida");
	DtosShortMsg.nTimer = 0;
}

int DtosInit()
{
	if (DtoDD == NULL || bDrvOkay == 0) {
		return 1;
	}

	// Get the game screen size
	BurnDrvGetScreen(&nDtosWidth, &nDtosHeight);

	RECT rect = { 0, 0, 0, 0 };
	GetClientScreenRect(hVidWnd, &rect);
	if (bPanelDlgIsShown) {
		rect.top += nPanelHeight;
	}
	if (!bDtoStretch) {
		IntegerScale(&rect, nDtosWidth, nDtosHeight);
	}
	if (bVidScanlines && (rect.bottom - rect.top >= 2 * nDtosHeight)) {
		bDtosScan = 1;
	} else {
		bDtosScan = 0;
	}

	nDtosDepth = GetSurfaceDepth(DtoPrim);		// Get color depth of primary surface
	nDtosBpp = (nDtosDepth + 7) >> 3;
	nBurnBpp = nDtosBpp;						// Set Burn library Bytes per pixel

	// Make the mask to mask out all but the lowest intensity bit
	nHalfMask = 0;
	if (nDtosDepth == 15) {
		nHalfMask = 0xfbdefbde;
	} else {
		if (nDtosDepth == 16) {
			nHalfMask = 0xf7def7de;
		} else {
			nHalfMask = 0xfefefefe;
		}
	}

	// Use our callback to get colors:
	SetBurnHighCol(nDtosDepth);
	// Recalculate the whole palette next time BurnDraw is called
	// This will use our callback for the whole palette
	BurnRecalcPal();

	// Make the normal memory buffer
	if (DtosMakeMem()) {
		DtosExit();
		return 1;
	}

	// Make the DirectDraw secondary surface
	if (DtosMakeSurf()) {
		DtosExit();
		return 1;
	}

	DtosShortMsgInit();

	return 0;
}

void DtosShortMsgExit()
{
	if (ShortMsgFont) {
		DeleteObject(ShortMsgFont);
		ShortMsgFont = NULL;
	}
}

int DtosExit()
{
	RELEASE(pddsDtos)
	nDtosMemPitch = 0;
	if (DtosMem) {
		free(DtosMem);
		DtosMem = NULL;
	}

	nHalfMask = 0; nDtosBpp = 0; nDtosDepth = 0;
	nDtosWidth = 0; nDtosHeight = 0;
	bDtosScan = 0;

	nBurnBpp = 0; // Blank the Burn library Bytes per pixel

	// Destroy resources allocated for "OSD"
	DtosShortMsgExit();

	return 0;
}

// Copy DtosMem to pddsDtos
static int DtosMemToSurf(int nField,int nHalf)
{
	DDSURFACEDESC ddsd;

	unsigned char *pd, *ps;
	unsigned char *Surf;
	int nPitch;

	if (pddsDtos == NULL) {
		return 1;
	}

	// Lock the surface so we can write to it
	memset(&ddsd, 0, sizeof(ddsd));

	ddsd.dwSize = sizeof(ddsd);
	pddsDtos->Lock(NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL);
	if (ddsd.lpSurface == NULL) {
		return 1;
	}

	Surf = (unsigned char *)ddsd.lpSurface;
	nPitch = ddsd.lPitch;
	if (bDtosScan) {
		if (nField) {			// copy to odd fields
			Surf += nPitch;
		}
		nPitch <<= 1;
	}

	pd = Surf; ps = DtosMem;
	for (int y = 0; y < nDtosHeight; y++, pd += nPitch, ps += nDtosMemPitch) {
		if (nHalf == 0) {
			memcpy(pd, ps, nDtosMemPitch);
		} else {
			unsigned char* psEnd = ps + nDtosMemPitch;
			unsigned char* pdp = pd;
			unsigned char* psp = ps;

			do {
				unsigned int t;
				t = *((unsigned int *)(psp));
				t = (t & nHalfMask) >> 1;
				*((unsigned int *)(pdp)) = t;

				psp += 4;
				pdp += 4;
			} while (psp < psEnd);
		}
	}

	pddsDtos->Unlock(NULL);
	return 0;
}

// Run one frame and render the screen
int DtosFrame(bool bRedraw)			// bRedraw = 0
{
	if (DtosMem == NULL) {
		return 1;
	}

	// delayed 50% scanlines:
	if (bDtosScan && bVidScanHalf && bVidScanDelay) {
		DtosMemToSurf(1, 1);
	}

	pSShotBuffer = NULL;
	pBurnDraw = DtosMem;
	nBurnPitch = nDtosMemPitch;

	if (bDrvOkay) {
		if (bRedraw) {				// Redraw current frame
			if (BurnDrvRedraw()) {
				BurnDrvFrame();		// No redraw function provided, advance one frame
			}
		} else {
			BurnDrvFrame();			// Run one frame and draw the screen
		}
	} else {
		// Blank the screen
		memset(DtosMem, 0, nDtosMemPitch * nDtosHeight);
	}
	pSShotBuffer = pBurnDraw;
	pBurnDraw = NULL;
	nBurnPitch = 0;

	// non-delayed 50% scanlines:
	if (bDtosScan && bVidScanHalf && bVidScanDelay == 0) {
		DtosMemToSurf(1, 1);
	}

	DtosMemToSurf(0, 0);			// Copy the memory buffer to the directdraw buffer for later blitting
	return 0;
}

void DtosDisplayShortMsg()
{
	if (DtosShortMsg.nTimer) {
		HDC hDC;
		HFONT hFont;
		int nWidth, nHeight;

		IDirectDrawSurface_GetDC(pddsDtos, &hDC);
		SetBkMode(hDC, TRANSPARENT);
		hFont = (HFONT)SelectObject(hDC, ShortMsgFont);
		SetTextAlign(hDC, TA_TOP | TA_RIGHT);

		BurnDrvGetScreen(&nWidth, &nHeight);

		// Print a black shadow first
		SetTextColor(hDC, 0);
		TextOut(hDC, nWidth - 2 + 1, 2 + (1 << bDtosScan), DtosShortMsg.pMsgText, strlen(DtosShortMsg.pMsgText));
		// Print the text on top
		SetTextColor(hDC, DtosShortMsg.nColour);
		TextOut(hDC, nWidth - 2, 2, DtosShortMsg.pMsgText, strlen(DtosShortMsg.pMsgText));

		// Clean up
		SelectObject(hDC, hFont);
		IDirectDrawSurface_ReleaseDC(pddsDtos, hDC);

		// Switch off message display when the message has been displayed long enough
		if (GetCurrentFrame() > DtosShortMsg.nTimer) {
			DtosShortMsg.nTimer = 0;
		}
	}
}

// Paint the Dtos surface onto the primary surface
int DtosPaint(int bValidate)
{
	RECT Dest = { 0, 0, 0, 0 };

	if (DtoPrim == NULL || pddsDtos == NULL) {
		return 1;
	}

	GetClientScreenRect(hVidWnd, &Dest);
	if (bPanelDlgIsShown) Dest.top += nPanelHeight;

	if (bVidArcaderes && nVidFullscreen) {
		Dest.left = (Dest.right + Dest.left) / 2;
		Dest.left -= nDtosWidth / 2;
		Dest.right = Dest.left + nDtosWidth;

		Dest.top = (Dest.top + Dest.bottom) / 2;
		Dest.top -= nDtosHeight / 2;
		Dest.bottom = Dest.top + nDtosHeight;
	} else {
		if (!bDtoStretch) {
			IntegerScale(&Dest, nDtosWidth, nDtosHeight);
		}
	}

	if (pddsDtos->IsLost()) {		// Restore surface if lost
		pddsDtos->Restore();
	}
	if (DtoPrim->IsLost()) {		// Restore surface if lost
		DtoPrim->Restore();
		if (nVidFullscreen) {		// Clear the edges
			DtoPrimClear();
		}
	}

	DtosDisplayShortMsg();

	if (DtoBack != NULL) {
		// Triple bufferring
		if (DtoBack->Blt(&Dest, pddsDtos, NULL, DDBLT_WAIT, NULL) < 0) {
			return 1;
		}
		DtoPrim->Flip(NULL, DDFLIP_WAIT);
	} else {
		// Normal
		if (DtoPrim->Blt(&Dest, pddsDtos, NULL, DDBLT_WAIT, NULL) < 0) {
			return 1;
		}
	}

	if (bValidate) {
		// Validate the rectangle we just drew
		POINT c = {0, 0};
		ClientToScreen(hVidWnd, &c);
		Dest.left -= c.x; Dest.right -= c.x;
		Dest.top -= c.y; Dest.bottom -= c.y;
		ValidateRect(hVidWnd, &Dest);
	}

	return 0;
}

int DtosNewShortMsg(const char* pText, int nRGB, int nDuration, int nPriority)	// int nRGB = 0, int nDuration = 0, int nPriority = 0
{
	// If a message with a higher priority is being displayed, exit.
	if (DtosShortMsg.nTimer && DtosShortMsg.nPriority > nPriority) {
		return 1;
	}

	int nSize = strlen(pText);
	if (nSize > 31) {
		nSize = 31;
	}
	strncpy(DtosShortMsg.pMsgText, pText, nSize);
	DtosShortMsg.pMsgText[nSize] = 0;
	if (nRGB) {
		// Convert RGB value to COLORREF
		DtosShortMsg.nColour = RGB((nRGB >> 16), ((nRGB >> 8) & 0xFF), (nRGB & 0xFF));
	} else {
		// Default message colour (yellow)
		DtosShortMsg.nColour = RGB(0xFF, 0xFF, 0x7F);
	}
	if (nDuration) {
		DtosShortMsg.nTimer = GetCurrentFrame() + nDuration;
	} else {
		DtosShortMsg.nTimer = GetCurrentFrame() + 120;
	}
	DtosShortMsg.nPriority = nPriority;

	return 0;
}
