// Misc functions module
#include <math.h>
#include "app.h"

int bDoGamma = 0;
float nGamma = 1.3;

static unsigned char GammaLUT[256];

// Set the current directory to be the application's directory
int AppDirectory()
{
	char *szCmd = NULL;
	char *szPath = NULL;
	int nLen = 0;
	char *pc1, *pc2;
	szCmd = GetCommandLine();

	// Find the end of the "c:\directory\program.exe" bit
	if (szCmd[0] == '\"') {						// Filneame is enclosed in quotes
		szCmd++;
		for (pc1 = szCmd; *pc1; pc1++) {
			if (*pc1 == '\"') break;			// Find the last "
		}
	} else {
		for (pc1 = szCmd; *pc1; pc1++) {
			if (*pc1 == ' ') break;				// Find the first space
		}
	}
	// Find the last \ or /
	for (pc2 = pc1; pc2 >= szCmd; pc2--) {
		if (*pc2 == '\\') break;
		if (*pc2 == '/') break;
	}

	// Copy the name of the executable into a variable
	nLen = pc1 - pc2 - 1;
	if (nLen > EXE_NAME_SIZE) {
		nLen = EXE_NAME_SIZE;
	}
	memcpy(&szAppExeName, pc2 + 1, nLen);
	szAppExeName[nLen] = 0;

	// strip .exe
	if ((pc1 = strchr(szAppExeName, '.'))) {
		*pc1 = 0;
	}

	nLen = pc2 - szCmd;
	if (nLen <= 0) return 1;			// No path

	// Now copy the path into a new buffer
	szPath = (char *)malloc(nLen + 1);
	if (szPath == NULL) {
		return 1;
	}

	memcpy(szPath, szCmd + 1, nLen);
	szPath[nLen] = 0;
	SetCurrentDirectory(szPath);		// Finally set the current directory to be the application's directory
	free(szPath);

	return 0;
}

// This function takes a rectangle and scales it to either:
// - The largest possible multiple of both X and Y;
// - The largest possible multiple of  Y, modifying X to ensure the correct aspect ratio;
// - The window size
int IntegerScale(RECT *pRect, int nGameWidth, int nGameHeight)
{
	int xm, ym;										// The multiple of nScrnWidth and nScrnHeight we can fit in
	int nWidth, nHeight;
	int nScrnWidth, nScrnHeight;
	int nGameAspectX = 4, nGameAspectY = 3;
	nWidth = pRect->right - pRect->left;
	nHeight = pRect->bottom - pRect->top;

	if (bDrvOkay) {
		BurnDrvGetAspect(&nGameAspectX, &nGameAspectY);
	}

	xm = nWidth / nGameWidth;
	ym = nHeight / nGameHeight;

	extern RECT SystemWorkArea;

	if (nVidFullscreen) {
		nScrnWidth = nVidWidth;
		nScrnHeight = nVidHeight;
	} else {
		nScrnWidth = SystemWorkArea.right - SystemWorkArea.left;
		nScrnHeight = SystemWorkArea.bottom - SystemWorkArea.top;
	}

	if (bVidCorrectAspect && bVidScanlines && ym >= 2 && xm) {	// Correct aspect ratio with scanlines
		int nWidthScratch, nHeightScratch;
		int ymScratch = ym;
		do {
			nHeightScratch = nGameHeight * ymScratch;
			nWidthScratch = nHeightScratch * nMonAspectY * nGameAspectX * nScrnWidth / nScrnHeight / nMonAspectX / nGameAspectY;
			ymScratch--;
		} while (nWidthScratch > nWidth && ymScratch >= 2);
		if (nWidthScratch > nWidth) { 				// The image is too high
			if (nGameWidth < nGameHeight) {			// Vertical games
				nHeight = nWidth * nMonAspectY * nGameAspectY * nScrnWidth / nScrnHeight / nMonAspectX / nGameAspectX;
			} else {								// Horizontal games
				nHeight = nWidth * nMonAspectX * nGameAspectY * nScrnHeight / nScrnWidth / nMonAspectY / nGameAspectX;
			}
		} else {
			nWidth = nWidthScratch;
			nHeight = nHeightScratch;
		}
	} else {
		if (bVidCorrectAspect) {					// Correct aspect ratio
			int nWidthScratch;
			nWidthScratch = nHeight * nMonAspectY * nGameAspectX * nScrnWidth / nScrnHeight / nMonAspectX / nGameAspectY;
			if (nWidthScratch > nWidth) {				// The image is too wide
				if (nGameWidth < nGameHeight) {		// Vertical games
					nHeight = nWidth * nMonAspectY * nGameAspectY * nScrnWidth / nScrnHeight / nMonAspectX / nGameAspectX;
				} else {				   			// Horizontal games
					nHeight = nWidth * nMonAspectX * nGameAspectY * nScrnHeight / nScrnWidth / nMonAspectY / nGameAspectX;
				}
			} else {
				nWidth = nWidthScratch;
			}
		} else {
			if (xm && ym) {							// Don't correct aspect ratio
				if (nVidFullscreen && nScrnWidth < 1024) {
					if (xm > ym) {
						xm = ym;
					} else {
						ym = xm;
					}
					nWidth = nGameWidth * xm;
					nHeight = nGameHeight * ym;
				} else {
					if (xm > ym) {
						nWidth = nGameWidth * xm;
						nHeight = nGameHeight * ym;
					} else {
						nWidth = nGameWidth * xm;
						nHeight = nGameHeight * ym;
					}
				}
			}
		}
	}

	pRect->left = (pRect->right + pRect->left) / 2;
	pRect->left -= nWidth / 2;
	pRect->right = pRect->left + nWidth;

	pRect->top = (pRect->top + pRect->bottom) / 2;
	pRect->top -= nHeight / 2;
	pRect->bottom = pRect->top + nHeight;
	return 0;
}

// Get the position of the client area of a window on the screen
int GetClientScreenRect(HWND hWnd,RECT *pRect)
{
  POINT Corner={0,0};
  GetClientRect(hWnd,pRect);
  Corner.x=0; Corner.y=0;
  ClientToScreen(hWnd,&Corner);
  pRect->left+=Corner.x; pRect->right+=Corner.x;
  pRect->top+=Corner.y; pRect->bottom+=Corner.y;
  return 0;
}

void ComputeGammaLUT()
{
	for(int i = 0; i < 256; i++) {
		GammaLUT[i] = (unsigned char) ((float)255.0 * pow((i / 255.0), nGamma));
	}
}

// Standard callbacks for 8/16/24/32 bit color:
static unsigned int HighCol8 (int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t=(r+g+b)/3;
  return t;
}

static unsigned int HighCol15(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<7)&0x7c00; // 0rrr rr00 0000 0000
  t|=(g<<2)&0x03e0; // 0000 00gg ggg0 0000
  t|=(b>>3)&0x001f; // 0000 0000 000b bbbb
  return t;
}

static unsigned int HighCol16(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<8)&0xf800; // rrrr r000 0000 0000
  t|=(g<<3)&0x07e0; // 0000 0ggg ggg0 0000
  t|=(b>>3)&0x001f; // 0000 0000 000b bbbb
  return t;
}

// 24-bit/32-bit
static unsigned int HighCol24(int r,int g,int b,int i)
{
  unsigned int t; (void)i;
  t =(r<<16)&0xff0000;
  t|=(g<<8 )&0x00ff00;
  t|=(b    )&0x0000ff;

  return t;
}

static unsigned int HighCol15Gamma(int r, int g, int b, int i)
{
  unsigned int t; (void)i;
  t = (GammaLUT[r] << 7) & 0x7C00; // 0rrr rr00 0000 0000
  t |= (GammaLUT[g] << 2) & 0x03E0; // 0000 00gg ggg0 0000
  t |= (GammaLUT[b] >> 3) & 0x001F; // 0000 0000 000b bbbb
  return t;
}

static unsigned int HighCol16Gamma(int r, int g ,int b, int i)
{
  unsigned int t; (void)i;
  t = (GammaLUT[r] << 8) & 0xF800; // rrrr r000 0000 0000
  t |= (GammaLUT[g] << 3) & 0x07E0; // 0000 0ggg ggg0 0000
  t |= (GammaLUT[b] >> 3) & 0x001F; // 0000 0000 000b bbbb
  return t;
}

// 24-bit/32-bit
static unsigned int HighCol24Gamma(int r, int g, int b, int i)
{
  unsigned int t; (void)i;
  t = (GammaLUT[r] << 16) & 0xFF0000;
  t |= (GammaLUT[g] << 8) & 0x00FF00;
  t |= GammaLUT[b] & 0x0000FF;

  return t;
}

int SetBurnHighCol(int nDepth)
{
	if (nDepth<= 8) {
		BurnHighCol = HighCol8;
		return 0;
	}

	if (bDoGamma) {
		if (nDepth <= 15) {
			BurnHighCol = HighCol15Gamma;
			return 0;
		}
		if (nDepth <= 16) {
			BurnHighCol = HighCol16Gamma;
			return 0;
		}
  		BurnHighCol = HighCol24Gamma;
	} else {
		if (nDepth <= 15) {
			BurnHighCol = HighCol15;
			return 0;
		}
		if (nDepth <= 16) {
			BurnHighCol = HighCol16;
			return 0;
		}
  		BurnHighCol = HighCol24;
	}
  return 0;
}

// Put a window in the middle of another window
int WndInMid(HWND hMid,HWND hBase)
{
  RECT MidRect={0,0,0,0};  int mw=0,mh=0;
  RECT BaseRect={0,0,0,0}; int bx=0,by=0;
  // Find the height and width of the Mid window
  GetWindowRect(hMid,&MidRect);
  mw=MidRect.right-MidRect.left;
  mh=MidRect.bottom-MidRect.top;

  // Find the center of the Base window
  GetWindowRect(hBase,&BaseRect);
  bx=BaseRect.left+BaseRect.right; bx>>=1;
  by=BaseRect.top+BaseRect.bottom; by>>=1;

  // Center the window
  SetWindowPos(hMid,NULL,bx-(mw>>1),by-(mh>>1),0,0,SWP_NOSIZE | SWP_NOZORDER);
  return 0;
}

static char szKeyName[0x54][16]=
{
  //0x00:
  "","Escape","1","2","3","4","5","6","7","8","9","0","Minus","Equals","Backspace",
  // 0x0f
  "Tab","Q","W","E","R","T","Y","U","I","O","P","[","]","Enter",
  // 0x1d
  "Left Ctrl","A","S","D","F","G","H","J","K","L","Semicolon","Apostrophe","Grave",
  // 0x2a
  "Left Shift","Backslash","Z","X","C","V","B","N","M","Comma","Period","Slash","Right Shift",
  // 0x37
  "NumPad *","Left Alt","Space","Caps Lock",
  // 0x3b
  "F1","F2","F3","F4","F5","F6","F7","F8","F9","F10",
  // 0x45
  "NumLock","ScrollLock",
  "NumPad 7","NumPad 8","NumPad 9","NumPad -",
  "NumPad 4","NumPad 5","NumPad 6","NumPad +",
  "NumPad 1","NumPad 2","NumPad 3",
  "NumPad 0","NumPad ."
};

// Print a nice name for input code c
char *InputCodeDesc(int c)
{
  static char szString[64];
  char *szName="";
  if (c>=0x4000)
  {
    int nJoy=0,nCode=0;
    nJoy=(c>>8)&0x3f;
    nCode=c&0xff;
    if (nCode>=0x10)
    {
      sprintf (szString,"Joy-%d Button %d",nJoy+1,nCode-0x0f);
      return szString;
    }
    if (nCode>=1 && nCode<=4)
    {
      static char szDir[4][8]={"Up","Down","Left","Right"};
      sprintf (szString,"Joy-%d %s",nJoy+1,szDir[nCode-1]);
      return szString;
    }
  }

  // Try and get a name for the key code
  if (c<0x54) szName=szKeyName[c]; // Get from the table of first few keys
  else if (c==0x56) szName="\\";
  else if (c==0x57) szName="F11";
  else if (c==0x58) szName="F12";
  else if (c==0x9C) szName="NumPad Enter";
  else if (c==0x9D) szName="Right Ctrl";
  else if (c==0xB5) szName="NumPad /";
  else if (c==0xB7) szName="PrintScrn";
  else if (c==0xB8) szName="Right Alt";
  else if (c==0xC5) szName="Pause";
  else if (c==0xC7) szName="Home";
  else if (c==0xC8) szName="Up Arrow";
  else if (c==0xC9) szName="PgUp";
  else if (c==0xCB) szName="Left Arrow";
  else if (c==0xCD) szName="Right Arrow";
  else if (c==0xCF) szName="End";
  else if (c==0xD0) szName="Down Arrow";
  else if (c==0xD1) szName="PgDn";
  else if (c==0xD2) szName="Insert";
  else if (c==0xD3) szName="Delete";
  else if (c==0xDB) szName="Left Win";
  else if (c==0xDC) szName="Right Win";
  else if (c==0xDD) szName="AppMenu";

  if (szName[0]) sprintf (szString,"%s",szName);
  else sprintf (szString,"Key 0x%.2x",c);
  return szString;
}
