// Burner Panel dialog module
#include "app.h"

HMENU hMenu=NULL;
HWND hPanelDlg=NULL; // Handle to the Input Dialog
int bPanelDlgIsShown=0;
int nPanelHeight=0;
int bAutoPause=1;
static HWND hParent=NULL;

static BOOL OnInitDialog(HWND, HWND, LPARAM);
static void OnDestroy(HWND);
static void OnShowWindow(HWND, BOOL, UINT);
static void OnCommand(HWND, int, HWND, UINT);

static char szItemText[32];
static MENUITEMINFO menuItem = {
	sizeof(MENUITEMINFO),
	MIIM_TYPE,
	MFT_STRING,
	0,
	0,
	NULL,
	NULL,
	NULL,
	(int)hParent,
	szItemText,
	0,
};

// Update the arade resolution menuitem
void CreateArcaderesItem()
{
	int nGameWidth, nGameHeight;

	LoadString(hAppInst, ID_STRING + 0, szItemText, sizeof(szItemText));
	if (bDrvOkay) {
		BurnDrvGetScreen(&nGameWidth, &nGameHeight);
		sprintf(szItemText + strlen(szItemText), "\t(%i  %i)", nGameWidth, nGameHeight);
	}
	menuItem.cch = strlen(szItemText);
	SetMenuItemInfo(hMenu, MENU_RES_ARCADE, 0, &menuItem);
}

// Update the custom resolution menuitem
static void CreateOtherresItem(bool bCustom)
{
	LoadString(hAppInst, ID_STRING + 1, szItemText, sizeof(szItemText));
	if (bCustom) {
		sprintf(szItemText + strlen(szItemText), "\t(%i  %i)", nVidWidth, nVidHeight);
	}
	menuItem.cch = strlen(szItemText);
	SetMenuItemInfo(hMenu, MENU_RES_OTHER, 0, &menuItem);
}

static void	CreatepresetResItems()
{
	int i;
	for (i = 0; i < 4; i++) {
		sprintf(szItemText,"%i  %i", VidPreset[i].nWidth, VidPreset[i].nHeight);
		menuItem.cch = strlen(szItemText);
		SetMenuItemInfo(hMenu, MENU_RES_1 + i, 0, &menuItem);
	}
	return;
}

// Update the other gamma menuitem
static void CreateOtherGammaItem(bool bOther)
{
	LoadString(hAppInst, ID_STRING + 2, szItemText, sizeof(szItemText));
	if (bOther) {
		sprintf(szItemText + strlen(szItemText), "\t(%1.2f)", nGamma);
	}
	menuItem.cch = strlen(szItemText);
	SetMenuItemInfo(hMenu, MENU_GAMMA_OTHER, 0, &menuItem);
}

int PanelShow(int bShow)
{
  // Toggle panel dialog on and off
  if (bShow) { ShowWindow(hPanelDlg,SW_NORMAL); SetFocus(hPanelDlg); }
  else       { ShowWindow(hPanelDlg,SW_HIDE); }
  return 0;
}
static int PanelUpdate()
{
  CheckDlgButton(hPanelDlg,IDC_PANEL_STRETCH,bDtoStretch?BST_CHECKED:BST_UNCHECKED);
  CheckDlgButton(hPanelDlg,IDC_PANEL_TRIPLE,bDtoTripleBuffer?BST_CHECKED:BST_UNCHECKED);
  CheckDlgButton(hPanelDlg,IDC_PANEL_SCAN,  bVidScanlines?BST_CHECKED:BST_UNCHECKED);
  CheckDlgButton(hPanelDlg,IDC_PANEL_SOUND, (nBurnSoundChoice&1)?BST_CHECKED:BST_UNCHECKED);
  CheckDlgButton(hPanelDlg,IDC_PANEL_MUSIC, (nBurnSoundChoice&2)?BST_CHECKED:BST_UNCHECKED);
  return 0;
}



static BOOL CALLBACK DialogProc(HWND hDlg,UINT Msg,WPARAM wParam,LPARAM lParam)
{
  switch (Msg)
	  {
      HANDLE_MSG (hDlg, WM_INITDIALOG , OnInitDialog );
      HANDLE_MSGT(hDlg, WM_DESTROY    , OnDestroy    );
      HANDLE_MSGT(hDlg, WM_SHOWWINDOW , OnShowWindow );
      HANDLE_MSGT(hDlg, WM_COMMAND    , OnCommand    );
	  }

  return 0;
}


static BOOL OnInitDialog(HWND hDlg, HWND hwndFocus, LPARAM lInitParam)
{
	(void)hwndFocus; (void)lInitParam;

	RECT Panel;

    hParent = GetParent(hDlg);
    memset(&Panel,0,sizeof(Panel));
    GetWindowRect(hDlg,&Panel);
    nPanelHeight = Panel.bottom-Panel.top; // calculate our height
    bPanelDlgIsShown=0; // not shown yet
    return true; // 1 = we didn't set focus?

}

static void OnDestroy(HWND hDlg)
{
   (void)hDlg;

   bPanelDlgIsShown=0;
   hPanelDlg=NULL;
   hParent=NULL;

}

static void OnShowWindow(HWND hDlg, BOOL fShow, UINT status)
{
   (void)hDlg; (void)status;

   bPanelDlgIsShown=(int)fShow;
   if (bPanelDlgIsShown==0)  SetFocus(hParent); // If we are hidden, focus back on our parent

   InvalidateRect(hParent,NULL,1);
   UpdateWindow(hParent);

}

static void OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
    (void)hDlg; (void)hwndCtl;

    if(codeNotify == BN_CLICKED)
    {
       switch (id)
       {
          case IDCANCEL:
            ShowWindow(hPanelDlg,SW_HIDE);
            break;

          case IDC_PANEL_INPUT:
            // Toggle Input dialog
            if (hInpdDlg==NULL) InpdCreate();
            else DestroyWindow(hInpdDlg);
            break;

	      case DC_PANEL_FULLSCREEN:
            nVidFullscreen=!nVidFullscreen;
			PostMessage(NULL,WM_APP+0,0,0);
			bRunPause=0;
			break; // restart video

          case IDC_PANEL_STRETCH:  bDtoStretch=!bDtoStretch; PanelUpdate(); break;
          case IDC_PANEL_TRIPLE:   bDtoTripleBuffer=!bDtoTripleBuffer; PanelUpdate(); break;
          case IDC_PANEL_SCAN:
            bVidScanlines=!bVidScanlines; PanelUpdate();
            PostMessage(NULL,WM_APP+0,0,0); // restart video
            break;

          case IDC_PANEL_SOUND:  nBurnSoundChoice^=1; PanelUpdate(); break;
          case IDC_PANEL_MUSIC:  nBurnSoundChoice^=2; PanelUpdate(); break;

    	  case IDC_QUIT_NETPLAY:
            PostMessage(hScrnWnd,WM_QUIT,0,0);
			break;
     }
   }

}

void MenuUpdate()
{
	int var;

	CheckMenuItem(hMenu, MENU_PAUSE , bAltPause?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_ALLRAM , bDrvSaveAll?MF_CHECKED:MF_UNCHECKED);
	if (bDtoStretch) {
		var = MENU_STRETCH;
	} else {
		if (bVidCorrectAspect) {
			var = MENU_ASPECT;
		} else {
			var = MENU_NOSTRETCH;
		}
	}
	CheckMenuRadioItem(hMenu, MENU_NOSTRETCH, MENU_ASPECT, var, MF_BYCOMMAND);
	CheckMenuItem(hMenu, MENU_STRETCH, bDtoStretch?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_TRIPLE , bDtoTripleBuffer?MF_CHECKED:MF_UNCHECKED);
	CheckMenuRadioItem(hMenu, MENU_NORMAL, MENU_SCAN, bVidScanlines?MENU_SCAN:MENU_NORMAL, MF_BYCOMMAND);
	CheckMenuItem(hMenu, MENU_SCAN50 , bVidScanHalf?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_FLICKER, bVidScanDelay?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_RES_ARCADE, bVidArcaderes?MF_CHECKED:MF_UNCHECKED);

	if (nVidWidth==VidPreset[0].nWidth && nVidHeight==VidPreset[0].nHeight) {
		var = MENU_RES_1;
	} else {
		if (nVidWidth==VidPreset[1].nWidth && nVidHeight==VidPreset[1].nHeight) {
			var = MENU_RES_2;
		} else {
			if (nVidWidth==VidPreset[2].nWidth && nVidHeight==VidPreset[2].nHeight) {
				var = MENU_RES_3;
			} else {
				if (nVidWidth==VidPreset[3].nWidth && nVidHeight==VidPreset[3].nHeight) {
					var = MENU_RES_4;
				} else {
					var = MENU_RES_OTHER;
				}
			}
		}
	}
	if (var == MENU_RES_OTHER) {
		CreateOtherresItem(1);
	} else {
		CreateOtherresItem(0);
	}
	if (bVidArcaderes) {
		var = MENU_RES_ARCADE;
	}
	CheckMenuRadioItem(hMenu, MENU_RES_1, MENU_RES_OTHER, var, MF_BYCOMMAND);

	if(nVidDepth == 8) var = MENU_8;
	else if(nVidDepth == 16) var = MENU_16;
	else if(nVidDepth == 24) var = MENU_24;
	else var = MENU_32;
	CheckMenuRadioItem(hMenu, MENU_8, MENU_32, var, MF_BYCOMMAND);

	if (nGamma == (float)1.25) {
		var = MENU_GAMMA_DARKER;
	} else {
		if (nGamma == (float)0.80) {
			var = MENU_GAMMA_LIGHTER;
		} else {
			var = MENU_GAMMA_OTHER;
		}
	}
	if (var == MENU_GAMMA_OTHER) {
		CreateOtherGammaItem(1);
	} else {
		CreateOtherGammaItem(0);
	}
	if (!bDoGamma) {
		var = MENU_GAMMA_DO;
	}
	CheckMenuRadioItem(hMenu, MENU_GAMMA_DARKER, MENU_GAMMA_OTHER, var, MF_BYCOMMAND);
	CheckMenuItem(hMenu, MENU_GAMMA_DO, bDoGamma?MF_CHECKED:MF_UNCHECKED);

	CheckMenuItem(hMenu, MENU_SPRMSK, nBurnMask?MF_CHECKED:MF_UNCHECKED);
	if(nUseSysMemory == 1) var = MENU_SYSMEM;
	else if(nUseSysMemory == -1) var = MENU_MEMAUTO;
	else var = MENU_VIDEOMEM;
	CheckMenuRadioItem(hMenu, MENU_MEMAUTO, MENU_SYSMEM, var, MF_BYCOMMAND);
	CheckMenuItem(hMenu, MENU_SOUNDCODE, (nBurnSoundChoice&1)?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_MUSICCODE, (nBurnSoundChoice&2)?MF_CHECKED:MF_UNCHECKED);
	if(nDSoundSamRate == 11025) var = MENU_11025;
	else if(nDSoundSamRate == 22050) var = MENU_22050;
	else var = MENU_44100;
	CheckMenuRadioItem(hMenu, MENU_11025, MENU_44100, var, MF_BYCOMMAND);
	CheckMenuItem(hMenu, MENU_BASS, bAppBassFilter?MF_CHECKED:MF_UNCHECKED);
	CheckMenuItem(hMenu, MENU_AUTOPAUSE, bAutoPause?MF_CHECKED:MF_UNCHECKED);

	if (nAppThreadPriority == THREAD_PRIORITY_TIME_CRITICAL) var = MENU_PRIORITY_REALTIME;
	else if (nAppThreadPriority == THREAD_PRIORITY_HIGHEST) var = MENU_PRIORITY_HIGH;
	else if (nAppThreadPriority == THREAD_PRIORITY_ABOVE_NORMAL) var = MENU_PRIORITY_ABOVE_NORMAL;
	else if (nAppThreadPriority == THREAD_PRIORITY_BELOW_NORMAL) var = MENU_PRIORITY_BELOW_NORMAL;
	else if (nAppThreadPriority == THREAD_PRIORITY_LOWEST) var = MENU_PRIORITY_LOW;
	else var = MENU_PRIORITY_NORMAL;
	CheckMenuRadioItem(hMenu, MENU_PRIORITY_REALTIME, MENU_PRIORITY_LOW, var, MF_BYCOMMAND);
}

int MenuCreate(HWND hParent, int NetGame)
{
  if(NetGame)
  {
	SetMenu(hParent,NULL);

    if(hPanelDlg==NULL) hPanelDlg=CreateDialog(hAppInst,MAKEINTRESOURCE(IDD_PANELNEW),hParent,DialogProc);
    PanelUpdate();

  }
  else
  {
	if(hPanelDlg!=NULL) DestroyWindow(hPanelDlg);
    if(hMenu==NULL) hMenu=LoadMenu(hAppInst,MAKEINTRESOURCE(IDR_MENU));

	CreateArcaderesItem();
	CreatepresetResItems();

	MenuUpdate();
	if (bDrvOkay!=0) MenuLoadGame();
    nPanelHeight = GetSystemMetrics(SM_CYMENU);
    SetMenu(hParent, hMenu);
    DrawMenuBar(hParent);

  }

  return 0;
}

