// Driver Selector module
// TreeView Version by HyperYagami

// This module uses libpng from http://www.libpng.org

#ifdef _MSC_VER
 #pragma warning(disable:4611) // interaction between _setjmp and C++ destructors not portable
#endif

// Create a fake parent node if the clone is available but not the parent
#define CREATE_FAKE_PARENT 1
// Gary out non-available sets
#define GRAY_OUT_NON_AVAILABLE 1
// Expand non-available sets (so they won't hide the available clones)
#define EXPAND_NON_AVAILABLE_PARENTS 1
// Disable selecting non-available sets
#define DISABLE_NON_AVAILABLE_SELECT 0
// Preview directory
#define PREVIEW_DIRECTORY "previews\\"
// 1: PNG, 0: BMP
#define USE_PNG_SCREENSHOT 1
// Prompt user on loading non-working sets
#define NON_WORKING_PROMPT_ON_LOAD 1

#include "app.h"

#if USE_PNG_SCREENSHOT
 #include "png.h"
#endif

static HWND hSelDlg=NULL;
static HWND hSelList=NULL;
static int nDialogSelect=0; // The driver which this dialog selected

HBITMAP hBmp=NULL;
static HWND hTextLabel[4]={NULL,NULL,NULL,NULL}; // 4 things in our Info-box
static char TreeBuilding=0; // TVN_SELCHANGED blocker

// new finalburn.ini variables
int bLoadMenuAutoExpand = 0;
int nLoadMenuShowX = 0xff;

#define BIT_CPS1C 0
#define BIT_CPS2 1
#define BIT_SEGA 2
#define BIT_MISC 3
#define BIT_KONAMI 4
#define BIT_TOAPLAN 5

struct HardwareCodeBitLinkType {int nHardwareCode; int nBitLocation;};

HardwareCodeBitLinkType HardwareCodeBitLink[NO_OF_HARDWARE_CODES] = 
{
	{HARDWARE_MISC_MISC,BIT_MISC},
	{HARDWARE_CAPCOM_CPS1,BIT_CPS1C},
	{HARDWARE_CAPCOM_CPS1_QSOUND,BIT_CPS1C},
	{HARDWARE_CAPCOM_CPS1_GENERIC,BIT_CPS1C},
	{HARDWARE_CAPCOM_CPSCHANGER,BIT_CPS1C},
	{HARDWARE_CAPCOM_CPS2,BIT_CPS2},
	{HARDWARE_SEGA_AFTERBURNER,BIT_SEGA},
	{HARDWARE_SEGA_GALAXYFORCE,BIT_SEGA},
	{HARDWARE_SEGA_SYSTEM16A,BIT_SEGA},
	{HARDWARE_SEGA_SYSTEM16B,BIT_SEGA},
	{HARDWARE_SEGA_SYSTEM18,BIT_SEGA},
	{HARDWARE_SEGA_HANGON,BIT_SEGA},
	{HARDWARE_SEGA_SPACEHARRIER,BIT_SEGA},
	{HARDWARE_SEGA_OUTRUN,BIT_SEGA},
	{HARDWARE_KONAMI_TMNT,BIT_KONAMI},
	{HARDWARE_TOAPLAN_RAIZING,BIT_TOAPLAN},
};

struct NODEINFO
{
    int nBurnDrvNo;
    char bIsParent;
    HTREEITEM hTreeHandle;
    char* szRomName;
};

NODEINFO nBurnDrv[MAX_GAMES];
unsigned int nTmpDrvCount;

// Make a list view of the available drivers
static int SelListMake()
{

    int nOldSelect=nBurnDrvSelect; // preserve the currently selected driver
    unsigned int i,j;

    nTmpDrvCount=0;
    for(i=0;i<MAX_GAMES;i++) memset(&nBurnDrv[i],0,sizeof(nBurnDrv[i]));

    if (hSelList==NULL) return 1;

    // Add all the driver names to the list

    // 1st: parents
    for (i=0;i<nBurnDrvCount;i++)
    {
        nBurnDrvSelect=i; // Switch to driver i
        if (BurnDrvText(6)!=NULL) continue; // skip clones
        if (avOk && bAvbOnly && !gameAv[i]) continue;
        for (j=0;j<NO_OF_HARDWARE_CODES;j++)
			if (BurnDrvGetHardwareCode()==HardwareCodeBitLink[j].nHardwareCode) break; // got system
		if (j!=NO_OF_HARDWARE_CODES)
		{
			if (!(nLoadMenuShowX&(1<<HardwareCodeBitLink[j].nBitLocation))) continue; // show this system?
		}
		else 
		{
			if (!(nLoadMenuShowX&(1<<BIT_MISC))) continue; // can find the code.  show misc?
		}

        TV_INSERTSTRUCT TvItem;
        memset(&TvItem,0,sizeof(TvItem));
        TvItem.item.mask=TVIF_TEXT;
        TvItem.hInsertAfter=TVI_SORT;
        TvItem.item.pszText=BurnDrvText(1);

        nBurnDrv[nTmpDrvCount].nBurnDrvNo=i;
        nBurnDrv[nTmpDrvCount].szRomName=BurnDrvText(0);
        nBurnDrv[nTmpDrvCount].hTreeHandle=(HTREEITEM)SendMessage(hSelList,TVM_INSERTITEM ,0,(LPARAM)&TvItem);
        nBurnDrv[nTmpDrvCount].bIsParent=1;
        nTmpDrvCount++;
    }

    // 2nd: clones
    for (i=0;i<nBurnDrvCount;i++)
    {
        nBurnDrvSelect=i; // Switch to driver i
        if (BurnDrvText(6)==NULL) continue; // skip parents
        if (avOk && bAvbOnly && !gameAv[i]) continue;
        for (j=0;j<NO_OF_HARDWARE_CODES;j++)
			if (BurnDrvGetHardwareCode()==HardwareCodeBitLink[j].nHardwareCode) break; // got system
		if (j!=NO_OF_HARDWARE_CODES)
		{
			if (!(nLoadMenuShowX&(1<<HardwareCodeBitLink[j].nBitLocation))) continue; // show this system?
		}
		else 
		{
			if (!(nLoadMenuShowX&(1<<BIT_MISC))) continue; // can find the code.  show misc?
		}

        TV_INSERTSTRUCT TvItem;
        memset(&TvItem,0,sizeof(TvItem));
        TvItem.item.mask=TVIF_TEXT;
        TvItem.hInsertAfter=TVI_SORT;
        TvItem.item.pszText=BurnDrvText(1);

        // find its parent's handle
        for (j=0;j<nTmpDrvCount;j++)
            if (nBurnDrv[j].bIsParent)
                if (!_stricmp(BurnDrvText(6),nBurnDrv[j].szRomName))
                    {TvItem.hParent=nBurnDrv[j].hTreeHandle; break;}
#if CREATE_FAKE_PARENT
        // find it's parent and add a grayed-out node onto the tree
        if (!TvItem.hParent)
        {
            char szTempName[8];
            strcpy(szTempName,BurnDrvText(6));
            int nTempBurnDrvSelect=nBurnDrvSelect;
            for (j=0;j<nBurnDrvCount;j++)
            {
                nBurnDrvSelect=j;
                if (!strcmp(szTempName,BurnDrvText(0)))
                {
                    TV_INSERTSTRUCT TempTvItem;
                    memset(&TempTvItem,0,sizeof(TempTvItem));
                    TempTvItem.item.mask=TVIF_TEXT;
                    TempTvItem.hInsertAfter=TVI_SORT;
                    TempTvItem.item.pszText=BurnDrvText(1);
                    nBurnDrv[nTmpDrvCount].nBurnDrvNo=j;
                    nBurnDrv[nTmpDrvCount].szRomName=BurnDrvText(0);
                    nBurnDrv[nTmpDrvCount].hTreeHandle=(HTREEITEM)SendMessage(hSelList,TVM_INSERTITEM ,0,(LPARAM)&TempTvItem);
                    nBurnDrv[nTmpDrvCount].bIsParent=1;
                    TvItem.hParent=nBurnDrv[nTmpDrvCount].hTreeHandle;
                    nTmpDrvCount++;
                    break;
                }
            }
            nBurnDrvSelect=nTempBurnDrvSelect;
        }
#endif
        nBurnDrv[nTmpDrvCount].nBurnDrvNo=i;
        nBurnDrv[nTmpDrvCount].szRomName=BurnDrvText(0);
        nBurnDrv[nTmpDrvCount].hTreeHandle=(HTREEITEM)SendMessage(hSelList,TVM_INSERTITEM ,0,(LPARAM)&TvItem);
        nTmpDrvCount++;
    }

#if EXPAND_NON_AVAILABLE_PARENTS && CREATE_FAKE_PARENT
    for (i=nTmpDrvCount;i>0;i--)
    {
        if ((nBurnDrv[i-1].bIsParent)&&(bLoadMenuAutoExpand||!gameAv[nBurnDrv[i-1].nBurnDrvNo]))
            SendMessage(hSelList,TVM_EXPAND,TVE_EXPAND,(LPARAM)nBurnDrv[i-1].hTreeHandle);
    }
#else
    if (bLoadMenuAutoExpand)
        for (i=nTmpDrvCount;i>0;i--)
            if (nBurnDrv[i-1].bIsParent) SendMessage(hSelList,TVM_EXPAND,TVE_EXPAND,(LPARAM)nBurnDrv[i-1].hTreeHandle);
#endif

    nBurnDrvSelect=nOldSelect;
    return 0;
}

static void SelOkay()
{
    // User clicked ok for a driver in the list
    unsigned int nSelect=0;
    HTREEITEM hSelectHandle = (HTREEITEM)SendMessage(hSelList, TVM_GETNEXTITEM, TVGN_CARET, ~0U);
    if (!hSelectHandle) return; // Selected nothing, return without closing the window
    // Search through nBurnDrv[] for the nBurnDrvNo according to the returned hSelectHandle
    for (unsigned int i=0;i<nTmpDrvCount;i++)
        if (hSelectHandle==nBurnDrv[i].hTreeHandle)
        {
            nSelect=nBurnDrv[i].nBurnDrvNo;
            break;
        }

#if DISABLE_NON_AVAILABLE_SELECT
    if (!gameAv[nSelect]) return; // Game not available, return without closing the window
#endif

#if NON_WORKING_PROMPT_ON_LOAD
	int nOldnBurnDrvSelect=nBurnDrvSelect;
	nBurnDrvSelect=nSelect;
	bool bStatus=BurnDrvIsWorking();
	nBurnDrvSelect=nOldnBurnDrvSelect;
	if (bStatus==FALSE)
	{
		int nRet=MessageBox(hSelDlg,"This game isn't working.  Load it anyway?","Warning!",
				MB_YESNO | MB_DEFBUTTON2 | MB_ICONWARNING);
		if (nRet==IDNO) return; // don't do it
	}
#endif

    nDialogSelect=nSelect;
    if (hBmp) {DeleteObject((HGDIOBJ)hBmp); hBmp=NULL;}
    EndDialog(hSelDlg,0);
}

static void RefreshPanel()
{
     // clear preview shot
    if (hBmp) {DeleteObject((HGDIOBJ)hBmp); hBmp=NULL;}
    SendDlgItemMessage(hSelDlg, IDC_SCREENSHOT, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)NULL);

#define cdb(aaa) CheckDlgButton(hSelDlg,IDC_CHECK##aaa,(nLoadMenuShowX&(1<<BIT_##aaa))?BST_CHECKED:BST_UNCHECKED);
    cdb(CPS1C)
    cdb(CPS2)
    cdb(SEGA)
    cdb(MISC)
    cdb(KONAMI)
    cdb(TOAPLAN)
#undef cdb
    CheckDlgButton(hSelDlg,IDC_CHECKAUTOEXPAND,bLoadMenuAutoExpand?BST_CHECKED:BST_UNCHECKED);
    CheckDlgButton(hSelDlg,IDC_CHECKAVAILABLEONLY,bAvbOnly?BST_CHECKED:BST_UNCHECKED);
}

static void RebuildEverything()
{
    RefreshPanel();
    TreeBuilding=1;
    SendMessage(hSelList,WM_SETREDRAW ,(WPARAM)(FALSE),(LPARAM)TVI_ROOT); // disable redraw
    SendMessage(hSelList,TVM_DELETEITEM ,0,(LPARAM)TVI_ROOT); // Destory all nodes
    SelListMake();
    SendMessage(hSelList,WM_SETREDRAW ,(WPARAM)(TRUE),(LPARAM)TVI_ROOT); // enable redraw
    // Clear the things in our Info-box
#define gdism(a) \
SendMessage(hTextLabel[##a], WM_SETTEXT, (WPARAM)0, (LPARAM)"");
    gdism(0)
    gdism(1)
    gdism(2)
    gdism(3)
#undef gdism
    TreeBuilding=0;
}

#if USE_PNG_SCREENSHOT

#define PNG_SIG_CHECK_BYTES 8
#define PNG_XSIZE_HORI 160
#define PNG_YSIZE_HORI 120
#define PNG_XSIZE_VERT PNG_YSIZE_HORI
#define PNG_YSIZE_VERT PNG_XSIZE_HORI
#define PNG_BITDEPTH_24BIT 24

typedef struct tagIMAGE {
    LONG    width;
    LONG    height;
    DWORD   rowbytes;
    DWORD   imgbytes;
    BYTE    **rowptr;
    BYTE    *bmpbits;
} IMAGE;

void img_free( IMAGE *img )
{
    free(img->rowptr);
    free(img->bmpbits);
}

int img_alloc( IMAGE *img )
{
    BYTE *bp,**rp;
    LONG n;

    img->rowbytes=((DWORD)img->width * PNG_BITDEPTH_24BIT + 31) / 32 * 4;
    img->imgbytes=img->rowbytes * img->height;
    img->rowptr=(BYTE**)malloc((size_t)img->height * sizeof(BYTE*));
    img->bmpbits=(BYTE*)malloc((size_t)img->imgbytes);

    if(img->rowptr==NULL || img->bmpbits==NULL) { img_free(img); return 0; }

    n=img->height;
    rp=img->rowptr;
    bp=img->bmpbits;

    bp+=img->imgbytes;
    while(--n>=0)
    {
        ((DWORD *)bp)[-1]=0;
        bp-=img->rowbytes;
        *(rp++)=bp;
    }

    return 1;
}

HBITMAP LoadPNG(HWND hDlg, FILE* fp)
{
    IMAGE img;
    png_uint_32 width, height;
    int bit_depth, color_type;

    // check signature
    unsigned char pngsig[PNG_SIG_CHECK_BYTES];
    fread(pngsig, 1, PNG_SIG_CHECK_BYTES, fp);
    if (png_sig_cmp(pngsig, 0, PNG_SIG_CHECK_BYTES)) return 0;

    png_structp png_ptr=png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!png_ptr) return 0;

    png_infop info_ptr=png_create_info_struct(png_ptr);
    if (!info_ptr)
    {
        png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
        return 0;
    }

    if (setjmp(png_jmpbuf(png_ptr)))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
        return 0;
    }

    memset( &img, 0, sizeof(IMAGE) );
    png_init_io(png_ptr, fp);
    png_set_sig_bytes(png_ptr, PNG_SIG_CHECK_BYTES);
    png_read_info( png_ptr, info_ptr );
    png_get_IHDR( png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL );

    // we want our pics to be either 160x120x24b or 120x160x24b
    img.width=(LONG)width;
    img.height=(LONG)height;
    if (!((img.width==PNG_XSIZE_HORI && img.height==PNG_YSIZE_HORI) ||
        (img.width==PNG_XSIZE_VERT && img.height==PNG_YSIZE_VERT)))
        longjmp(png_ptr->jmpbuf, 1);
    if (color_type!=PNG_COLOR_TYPE_RGB && color_type!=PNG_COLOR_TYPE_RGB_ALPHA)
        longjmp(png_ptr->jmpbuf, 1);

    // initialize our img struct (row pointers and such)
    if(!img_alloc(&img)) longjmp( png_ptr->jmpbuf, 1 );

    // after this, if bad things happen in libpng we wanna img_free(&img) as well
    if (setjmp(png_jmpbuf(png_ptr)))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
        img_free(&img);
        return 0;
    }

    // read our thing
    png_set_bgr(png_ptr);
    png_read_update_info(png_ptr, info_ptr);
    png_read_image(png_ptr, img.rowptr);
    png_read_end(png_ptr, (png_infop)NULL);
    png_destroy_read_struct( &png_ptr, &info_ptr, (png_infopp)NULL);

    // create a bitmap
    BITMAPINFO* bi;
    bi=(BITMAPINFO*)LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER)+sizeof(RGBQUAD));
    if (!bi) {img_free(&img); return 0;}

    bi->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
    bi->bmiHeader.biWidth=img.width;
    bi->bmiHeader.biHeight=img.height;
    bi->bmiHeader.biPlanes=1;
    bi->bmiHeader.biBitCount=PNG_BITDEPTH_24BIT;
    bi->bmiHeader.biCompression=BI_RGB;
    bi->bmiHeader.biSizeImage=img.imgbytes;
    bi->bmiHeader.biXPelsPerMeter=0;
    bi->bmiHeader.biYPelsPerMeter=0;
    bi->bmiHeader.biClrUsed=0;
    bi->bmiHeader.biClrImportant=0;

    HDC hDC;
    hDC=GetDC(hDlg);
    BYTE* pBits=NULL;
    HBITMAP hBmp=CreateDIBSection(hDC, (BITMAPINFO*)bi, DIB_RGB_COLORS, (void**)&pBits, NULL, 0);
    if (pBits) {memcpy(pBits, img.bmpbits, img.imgbytes);}
    ReleaseDC(hDlg, hDC);
    LocalFree(bi);
    img_free(&img);
    return hBmp;
}
#endif

static BOOL CALLBACK DialogProc(HWND hDlg,UINT Msg,WPARAM wParam,LPARAM lParam)
{
    if (Msg==WM_INITDIALOG)
    {
        hSelDlg=hDlg;
        SetWindowText(hSelDlg,APP_TITLE " - Select Game");
        hSelList=GetDlgItem(hSelDlg,IDC_TREE1);
        hTextLabel[0]=GetDlgItem(hSelDlg,IDC_TEXTROMNAME);
        hTextLabel[1]=GetDlgItem(hSelDlg,IDC_TEXTPARENT);
        hTextLabel[2]=GetDlgItem(hSelDlg,IDC_TEXTSYSTEM);
        hTextLabel[3]=GetDlgItem(hSelDlg,IDC_TEXTCOMMENT);
        RefreshPanel();
        TreeBuilding=1;
        SendMessage(hSelList,WM_SETREDRAW ,(WPARAM)(FALSE),(LPARAM)TVI_ROOT);
        SelListMake();
        SendMessage(hSelList,WM_SETREDRAW ,(WPARAM)(TRUE),(LPARAM)TVI_ROOT);
        TreeBuilding=0;
#ifndef _DEBUG
        ShowWindow(GetDlgItem(hSelDlg,IDC_CHECKSEGA),SW_HIDE);
        ShowWindow(GetDlgItem(hSelDlg,IDC_CHECKMISC),SW_HIDE);
        ShowWindow(GetDlgItem(hSelDlg,IDC_CHECKKONAMI),SW_HIDE);
        ShowWindow(GetDlgItem(hSelDlg,IDC_CHECKTOAPLAN),SW_HIDE);
#endif
        return 1;
    }

    if (Msg==WM_COMMAND)
    {
        if (HIWORD(wParam)==BN_CLICKED)
        {
            int wID=LOWORD(wParam);
            if (wID==IDOK) SelOkay();
            if (wID==IDROM)
            {
                RomsDirCreate();
                RebuildEverything();
            }
            if (wID==IDRESCAN)
            {
                bRescanRoms=TRUE;
                TestRomsCreate();
                RebuildEverything();
            }

            if (wID==IDCANCEL) SendMessage(hDlg,WM_CLOSE,0,0); // cancel=close

#define ret(aaa) if (wID==IDC_CHECK##aaa) {nLoadMenuShowX^=(1<<BIT_##aaa); RebuildEverything();}
            ret(CPS1C)
            ret(CPS2)
            ret(SEGA)
            ret(MISC)
            ret(KONAMI)
            ret(TOAPLAN)
#undef ret
            if (wID==IDC_CHECKAVAILABLEONLY) {bAvbOnly=!bAvbOnly; RebuildEverything();}
            if (wID==IDC_CHECKAUTOEXPAND) {bLoadMenuAutoExpand=!bLoadMenuAutoExpand; RebuildEverything();}
        }
    }

    if (Msg==WM_CLOSE)
    {
        nDialogSelect=-1;
        if (hBmp) {DeleteObject((HGDIOBJ)hBmp); hBmp=NULL;}
        EndDialog(hDlg,0);
    }

    if (Msg==WM_NOTIFY)
    {
        NMHDR *pNmHdr; pNmHdr=(NMHDR *)lParam;
        if (pNmHdr->code==NM_DBLCLK)
        {
            SelOkay();
            // disable double-click node-expand
            SetWindowLong(hSelDlg, DWL_MSGRESULT, 1);
            return 1;
        }
#if GRAY_OUT_NON_AVAILABLE
        if (pNmHdr->code==NM_CUSTOMDRAW && LOWORD(wParam)==IDC_TREE1)
        {
            LPNMLVCUSTOMDRAW lplvcd = (LPNMLVCUSTOMDRAW)lParam;
            HTREEITEM hSelectHandle;
            unsigned int i;
            switch(lplvcd->nmcd.dwDrawStage)
            {
                case CDDS_PREPAINT:
                    SetWindowLong(hSelDlg, DWL_MSGRESULT, CDRF_NOTIFYITEMDRAW);
                    return 1;
                case CDDS_ITEMPREPAINT:
                    hSelectHandle = (HTREEITEM)(lplvcd->nmcd.dwItemSpec);
                    // Gary out the non-selected non-available treeitems
                    TV_ITEM TvItem;
                    TvItem.hItem=hSelectHandle;
                    TvItem.mask=TVIF_STATE;
                    SendMessage(hSelList,TVM_GETITEM,0,(LPARAM)&TvItem);
                    if (TvItem.state&TVIS_SELECTED)
                    {
                        SetWindowLong(hSelDlg, DWL_MSGRESULT, CDRF_DODEFAULT);
                        return 1;
                    }
                    for (i=0;i<nTmpDrvCount;i++)
                        if (hSelectHandle==nBurnDrv[i].hTreeHandle)
                        {
                            if (!gameAv[nBurnDrv[i].nBurnDrvNo]) lplvcd->clrText=RGB(192,192,192);
                            break;
                        }
                    SetWindowLong(hSelDlg, DWL_MSGRESULT, CDRF_NEWFONT);
                    return 1;
                default:
                    SetWindowLong(hSelDlg, DWL_MSGRESULT, CDRF_DODEFAULT);
                    return 1;
            }
        }
#endif
        if (pNmHdr->code==TVN_SELCHANGED && !TreeBuilding)
        {
            int nOldSelect=nBurnDrvSelect;
            HTREEITEM hSelectHandle = (HTREEITEM)SendMessage(hSelList, TVM_GETNEXTITEM, TVGN_CARET, ~0U);
            // Search through nBurnDrv[] for the nBurnDrvNo according to the returned hSelectHandle
            for (unsigned int i=0;i<nTmpDrvCount;i++)
                if (hSelectHandle==nBurnDrv[i].hTreeHandle)
                {
                    nBurnDrvSelect=nBurnDrv[i].nBurnDrvNo;
                    break;
                }
// Suck the text out of the drivers via BurnDrvText(i)
#define gdism(a,b) \
SendMessage(hTextLabel[##b], WM_SETTEXT, (WPARAM)0, (LPARAM)BurnDrvText(##a));
                gdism(0,0)
                gdism(6,1)
                gdism(4,2)
                gdism(2,3)
#undef gdism

//Screen Shot Stuff*************************************************

				if (hBmp) {DeleteObject((HGDIOBJ)hBmp); hBmp=NULL;}
#if !USE_PNG_SCREENSHOT
                char TempName[22];
                strcpy(TempName,PREVIEW_DIRECTORY);
                strcat(TempName,BurnDrvText(0));
                strcat(TempName,".bmp");
                hBmp = (HBITMAP) LoadImage(0, TempName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
                if (!hBmp && BurnDrvText(6))
                {
                    // have parent, will try
                    strcpy(TempName,PREVIEW_DIRECTORY);
                    strcat(TempName,BurnDrvText(6));
                    strcat(TempName,".bmp");
                    hBmp = (HBITMAP) LoadImage(0, TempName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
                }
#else
                hBmp=NULL;
                char TempName[22];
                strcpy(TempName,PREVIEW_DIRECTORY);
                strcat(TempName,BurnDrvText(0));
                strcat(TempName,".png");
                FILE *fp=fopen(TempName,"rb");
                if (!fp && BurnDrvText(6))
                {
                    // have parent, will try
                    strcpy(TempName,PREVIEW_DIRECTORY);
                    strcat(TempName,BurnDrvText(6));
                    strcat(TempName,".png");
                    fp=fopen(TempName,"rb");
                }
                if (fp)
                {
                    hBmp=LoadPNG(hDlg, fp);
                    fclose(fp);
                }
#endif
                SendDlgItemMessage(hDlg, IDC_SCREENSHOT, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)hBmp);
                nBurnDrvSelect=nOldSelect;
        }
    }
    return 0;
}

int SelDialog() // Choose a Burn driver
{
    int nRet=0;
    nRet=DialogBox(hAppInst,MAKEINTRESOURCE(IDD_SELNEW),hScrnWnd,DialogProc);
    hSelDlg=NULL; hSelList=NULL;
    return nDialogSelect;
}
