// State dialog module
#include "app.h"
#include <commdlg.h>

int bDrvSaveAll=0;

static char szChoice[MAX_PATH]=""; // File chosen by the user
static OPENFILENAME ofn;

static int MakeOfn()
{
  memset(&ofn,0,sizeof(ofn));
  ofn.lStructSize=sizeof(ofn);
  ofn.hwndOwner=hScrnWnd;
  ofn.lpstrFilter="Final Burn States\0*.fs\0\0";
  ofn.lpstrFile=szChoice;
  ofn.nMaxFile=sizeof(szChoice);
  ofn.lpstrInitialDir=".\\state";
  ofn.Flags=OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
  ofn.lpstrDefExt="fs";
  return 0;
}

// The automatic save
int StatedAuto(int bSave)
{
  int nRet=0; static char szName[32]="";
  sprintf (szName,"cfg\\%.8s.fs",BurnDrvText(0));
  if (bSave==0)
  {
    nRet=StateLoad(szName,bDrvSaveAll); // Load ram
    if (nRet!=0 && bDrvSaveAll) nRet=StateLoad(szName,0); // Couldn't get all - okay just try the nvram
  }
  else
  {
    nRet=StateSave(szName,bDrvSaveAll); // Save ram
  }
  return nRet;
}

static void CreateStateName(int nSlot)
{
	sprintf(szChoice, ".\\state\\%s %02x.fs", BurnDrvText(0), nSlot);
}

int StatedLoad(int nSlot)		// int nSlot = 0
{
	int nRet;
	int bOldPause;

	if (bDrvOkay == 0) {
		return 1;
	}

	if (nSlot) {
		CreateStateName(nSlot);
	} else {
		sprintf (szChoice, "%.8s*.fs", BurnDrvText(0));
		MakeOfn();
		ofn.lpstrTitle = "Load State";

		bOldPause = bRunPause;
		bRunPause = 1;
		nRet = GetOpenFileName(&ofn);
		bRunPause = bOldPause;

		if (nRet == 0) {		// Error
			return 1;
		}
	}

	nRet = StateLoad(szChoice,1);

	// Describe any possible errors:
	if (nRet == 3) {
		AppError("The save state is for the wrong game.", 0);
	} else {
		if (nRet == 4) {
			AppError("This state is too old and cannot be loaded.", 0);
		} else {
			if (nRet == 5) {
				AppError("Emulator is too old to load this state.", 0);
			} else {
				if (nRet) {
					AppError("Error loading state", 0);
				}
			}
		}
	}

	return 0;
}

int StatedSave(int nSlot)		// int nSlot = 0
{
	int bOldPause;

	if (bDrvOkay == 0) {
		return 1;
	}

	if (nSlot) {
 		CreateStateName(nSlot);
	} else {
		int nRet;

		sprintf (szChoice, "%.8s", BurnDrvText(0));
		MakeOfn();
		ofn.lpstrTitle = "Save State";
		ofn.Flags |= OFN_OVERWRITEPROMPT;

		bOldPause = bRunPause;
		bRunPause = 1;
		nRet = GetOpenFileName(&ofn);
		bRunPause = bOldPause;

  		if (nRet == 0) {		// Error
			return 1;
		}
	}

	if (StateSave(szChoice,1)) {
		AppError("Error saving state", 1);
	}

	return 0;
}
