#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <unistd.h>
#include "config.h"
#include "defaults.h"


int PrintRomsInfo = 0;
int FullScreen = def_FullScreen;
int Sound = def_Sound;
int SFilter = def_SFilter;
int SampleRate = def_SampleRate;
int Verbose = def_Verbose;
int ShowHelp = 0;
int BufferVideo = def_BufferVideo;
int OpenGLMode = def_OpenGLMode;
char *RomName = NULL;
char *RomsPath = def_RomsPath;


#define DISABLE(A) (  (Arg) && \
	( (!strcasecmp(Arg,"no")) || (!strcasecmp(Arg,"off")) )  )


int Param_FullScreen(char *Arg, int NArg)
{
  if (DISABLE(Arg))
  {
    FullScreen = 0;
    return 1+NArg;
  }

  FullScreen = 1;
  return 1;
}


int Param_BuffVideo(char *Arg, int NArg)
{
  if (DISABLE(Arg))
  {
    BufferVideo = 0;
    return 1+NArg;
  }

  BufferVideo = 1;
  return 1;
}


int Param_Sound(char *Arg, int NArg)
{
  if (DISABLE(Arg))
  {
    Sound = 0;
    return 1+NArg;
  }

  Sound = 1;
  return 1;
}


int Param_SFilter(char *Arg, int NArg)
{
/*  if (DISABLE(Arg))
  {
    SFilter = 0;
    return 1+NArg;
  }

  SFilter = LowPass_Cmc;
  return 1;*/

  if ((!Arg) || (strlen(Arg) == 0))
  {
    printf("-sfilter needs one argument!!!\n");
    exit(1);
  }

  SFilter = atoi(Arg);
  return 1+NArg;
}


int Param_SRate(char *Arg, int NArg)
{
  if ((!Arg) || (strlen(Arg) == 0))
  {
    printf("-srate needs one argument!!!\n");
    exit(1);
  }

  SampleRate = atoi(Arg);

  return 1+NArg;
}


extern int HasLetters(char *Str);

int Param_PRInfo(char *Arg, int NArg)
{
  if ((!Arg) || (HasLetters(Arg)))
  {
    PrintRomsInfo = 1;
    return 1;
  }

  PrintRomsInfo = atoi(Arg);
  return 1+NArg;
}


int Param_Verbose(char *Arg, int NArg)
{
  if ((!Arg) || (HasLetters(Arg)))
  {
    Verbose = 1;
    return 1;
  }

  Verbose = atoi(Arg);
  return 1+NArg;
}


int Param_RPath(char *Arg, int NArg)
{
  int Length;

  if ((!Arg) || (strlen(Arg) == 0))
  {
    printf("-romspath needs one argument!!!\n");
    exit(1);
  }

  Length = strlen(Arg);
  if (Arg[Length-1] != '/')
  {
    RomsPath = (char *)malloc(Length+2);
    sprintf(RomsPath,"%s/",Arg);
  }
  else
  {
    RomsPath = (char *)malloc(Length+1);
    sprintf(RomsPath,"%s",Arg);
  }

  return 1+NArg;
}


int Param_ABuffers(char *Arg, int NArg)
{
  if ((!Arg) || (strlen(Arg) == 0) || (HasLetters(Arg)))
  {
    printf("-abuffers needs one numeric argument!!!\n");
    exit(1);
  }

  nABuffers = atoi(Arg);
  if (nABuffers < 3) { nABuffers = 3; }

  return 1+NArg;
}


int Param_Help(char *Arg, int NArg)
{
  ShowHelp = 1;
  return 1;
}



struct {
	char *ParamNme;
	int (*ParamFunc)(char *Arg, int NArg);
	char *Descr;
} ParamTbl[] =
{
	{ "fullscreen", Param_FullScreen, "run fullscreen. Use \"-fullscreen=off\" to disable." },
	{ "buffvideo", Param_BuffVideo, "use software video buffers. (may reduce flicker) (Slow)" },
	{ "sound", Param_Sound, "run with sound. Use \"-sound=off\" to disable." },
	{ "sfilter", Param_SFilter, "select which sound filter to use. (numeric)" },
	{ "srate", Param_SRate, "set samplerate." },
//	{ "samplerate", Param_SRate, "alias to -srate." },
	{ "rominfo", Param_PRInfo, "print roms info with specified level of detail." },
	{ "verbose", Param_Verbose, "set emulator verbosenes level." },
	{ "romspath", Param_RPath, "set zipped roms location." },
	{ "abuffers", Param_ABuffers, "set number of audio buffers." },
	{ "help", Param_Help, "print help." },
};


int ProcesaParam(char *P1, char *P2)
{
  char *Param;
  int a, NA;

  if (*P1 != '-')
  {
    RomName = P1;
    return 1;
  }
  else
  {
    a = 0;
    while (a < (sizeof(ParamTbl) / sizeof(ParamTbl[0])) )
    {
      if (!strncasecmp(P1+1, ParamTbl[a].ParamNme, strlen(ParamTbl[a].ParamNme)))
      {
        NA = 0;
        Param = P1+1 + strlen(ParamTbl[a].ParamNme);
        while (*Param == '=') { Param++; }
        if (!(*Param)) { Param = P2; NA = 1; }
        return ParamTbl[a].ParamFunc(Param, NA);
      }
      a++;
    }
    fprintf(stderr,"\nUnrecognized parameter: %s\nTry -help\n",P1);
    exit(1);
  }
}


void ProcesaConfigFile(char *FleName)
{
  FILE *F;
  char Line[1024];
  char *Str2;
  char *Name, *Value;

  F = fopen(FleName, "rt");
  if (!F) { return; } 		// File not found or problem while opening

  while (fgets(Line+1, 1023, F))
  {
    // Remove newline characters:
    Str2 = strstr(Line+1, "\x0A");
    if (Str2) { *Str2 = 0; }
    Str2 = strstr(Line+1, "\x0D");
    if (Str2) { *Str2 = 0; }

    // Remove comments at end of line:
    Str2 = strstr(Line+1, ";");
    if (Str2) { *Str2 = 0; }

    // Gues if there is value or only an option name
    Str2 = strstr(Line+1, "=");

    if (Str2)
    {
      // There is option name AND value:
      *Str2 = 0;	// Replace the "=" by the string separator
      Str2++;

      Name = Line+1;
      Value = Str2;

      // Remove leading spaces:
      while ((*Name == ' ') || (*Name == 9)) { Name++; }
      *(--Name) = '-';		// Add option marker

      // Remove trailing spaces:
      Str2 = Name + strlen(Name)-1;
      while ((*Str2 == ' ') || (*Str2 == 9)) { Str2--; }
      *(Str2+1) = 0;

      // Remove leading spaces:
      while ((*Value == ' ') || (*Value == 9)) { Value++; }

      // Remove trailing spaces:
      Str2 = Value + strlen(Value)-1;
      while ((*Str2 == ' ') || (*Str2 == 9)) { Str2--; }
      *(Str2+1) = 0;

      if (strlen(Name) > 1)
      {
        ProcesaParam(Name, Value);
      }
    }
    else
    {
      // There is name only
      Name = Line+1;

      // Remove leading spaces:
      while ((*Name == ' ') || (*Name == 9)) { Name++; }
      *(--Name) = '-';		// Add option marker

      // Remove trailing spaces:
      Str2 = Name + strlen(Name)-1;
      while ((*Str2 == ' ') || (*Str2 == 9)) { Str2--; }
      *(Str2+1) = 0;

      if (strlen(Name) > 1)
      {
        ProcesaParam(Name, NULL);
      }
    }
  }

  fclose(F);
}


void ProcesaAllConfigFiles(void)
{
  char FleName[1024];
  struct passwd *PW;

  ProcesaConfigFile(def_SYSTEM_CONFIG);

  PW = getpwuid(getuid());	// Get current user info
  sprintf(FleName,"%s/%s",PW->pw_dir,def_USER_CONFIG);	// Build name
//  free(PW);	// Crashes if enabled.  :-m

  ProcesaConfigFile(FleName);
}


void PrintParamsHelp(void)
{
  int a;

  for (a = 0; a < sizeof(ParamTbl) / sizeof(ParamTbl[0]); a++)
  {
    printf("  -%-12s %s\n",ParamTbl[a].ParamNme, ParamTbl[a].Descr);
  }
}
