// DSP_ComaC module
// ----------------
//   by Daniel Moreno - ComaC (2001)  < comac2k@teleline.es >
//
//  This module just fakes the "dsp.c" functions of FinalBurn and uses
//  a IIR low pass filter, which is pretty faster and has a better
//  frequency response.


#include <stdlib.h>
#include "dsp2_cmc.h"
#include "lowpass3.hpp"

// NOTE: don't modify these defines unless you are 100% sure you know how
//   to deal with the parameters of a 2nd order resonant lowpass filter.
//   DO NOT EVEN MODIFY SampleFreq!!!!
//
//   BTW: if you are curious, these parameters imitate the behavour of
//   finalburn's "old" convolution filter.
// ----------------------------------------------------------------------
#define SampleFreq 44100.0

#define CutFreq 14000.0
#define Q 0.4
#define Gain 1.0

#define CutFreq2 1500.0
#define Q2 0.3
#define Gain2 1.475


class LowPass3 *LP2_1 = NULL, *LP2_2 = NULL;


int Dsp2Do(short *Buff, int Len)
{
  if ((!LP2_1) || (!LP2_2)) { return 1; }

  LP2_1->Filter(Buff, Len);	// Left
  LP2_2->Filter(Buff+1, Len);	// Right

  return 0;
}


extern int nBurnSoundRate;

int Dsp2Init(void)
{
  LP2_1 = new LowPass3(CutFreq2, nBurnSoundRate/*SampleFreq*/, Q2, Gain2);
  LP2_2 = new LowPass3(CutFreq2, nBurnSoundRate/*SampleFreq*/, Q2, Gain2);
  return 0;
}

int Dsp2Exit(void)
{
  delete LP2_1;
  delete LP2_2;

  LP2_1 = NULL;
  LP2_2 = NULL;

  return 0;
}
