#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include "app.h"
#include "burn.h"
#include "vidsdl.h"
#include <SDL.h>
#include "version.h"
#include "config.h"

#ifdef COMAC
#include "dsp_cmc.h"
#include "dsp2_cmc.h"
#endif

#undef main
#undef Player2


int scrWidth, scrHeight, BPPs, BytesPP;
unsigned long long int CurrTime, TimeFrame;
unsigned char *DrawBkp;


struct cmcInput
{
  int Key1, Key2;
  int Press1, Press2;
  int Type;
  int Value1, Value2, CenterVal;
  unsigned char *pVal;
} *Inputs;


inline unsigned long long int useconds(void)
{
  struct timeval Tmp;
  struct timezone Tmp2;

  gettimeofday(&Tmp, &Tmp2);

  return ((unsigned long long int)Tmp.tv_sec) * 1000000 + Tmp.tv_usec;
}


void prntRomsInfo(int detail)
{
  int a, b;
  char *Nombre, *N2;

  for (a = 0; a < nBurnDrvCount; a++)
  {
    nBurnDrvSelect = a;
    Nombre = BurnDrvText(1);
    printf("Rom n %3i: \"%s\"", a, Nombre);
      
    printf("  ( ");
    b = 0;
    BurnDrvGetZipName(&Nombre, b++);
      
    while (!BurnDrvGetZipName(&N2, b++))
    {
      printf("%s, ",Nombre);
      Nombre = N2;
    }
    printf("%s )\n", Nombre);
  }
}


void lnxAudioInit(void)
{
  if (Sound)
  {
    nBurnSoundRate = InitAudio(SampleRate, 60);
    if (!nBurnSoundRate)
    {
      if (Verbose) { fprintf(stderr,"\nProblems with audio. Running without sound. :(\n\n"); }
      Sound=0;
    }
  }

  if (Sound)
  {
    nBurnSoundLen = nBurnSoundRate / 60;
    pBurnSoundOut = (short *)malloc((nBurnSoundLen+1)*4);
    memset(pBurnSoundOut, 0, nBurnSoundLen*4);

#ifdef COMAC
    switch (SFilter)
    {
      case LowPass_Cmc: DspInit(); break;
      case LowPass2_Cmc: Dsp2Init(); break;
    }
#endif
  }
  else
  {
    nBurnSoundLen = 0;
    pBurnSoundOut = NULL;
  }
}


int HasLetters(char *Str)
{
  while (*Str)
  {
    if ((*Str < '0') || (*Str > '9')) { return 1; }
    Str++;
  }

  return 0;
}


int GetRomNumber(char *Str)
{
  int Tmp;

  if (!Str)
  {
    printf("No game specified.\n");
    exit(1);
  }

  if (HasLetters(Str))
  {
    nBurnDrvSelect = 0;
    while (  (nBurnDrvSelect < nBurnDrvCount) &&
             (strcasecmp(BurnDrvText(0),Str))  )
    {
      nBurnDrvSelect++;
    }

    Tmp = nBurnDrvSelect;

    if (Tmp >= nBurnDrvCount)
    {
      printf("Invalid game rom name: %s\n",Str);
      exit(1);
    }
  }
  else
  {
    Tmp = atoi(Str);
    if ((Tmp < 0) || (Tmp > nBurnDrvCount))
    {
      printf("Invalid game rom number: %s\n",Str);
      exit(1);
    }
  }

  return Tmp;
}


int main(int ArgN, char *Args[])
{
  int a, b, c, Salir = 0, LostFrames = 0;
  int Tecla, Valor;
  char *Nombre, *N2;
  char *BuffPantalla;
  struct BurnInputInfo Ii;
  unsigned long long int Tiempo, InitTime;
  int nInputs;
  
  BurnLibInit();
  
  printf("ComaC release version: %i for %s\n",CMC_VERSION, DEST_OS);
  printf("Emulator library version %i.%X\n",nBurnVer >> 24, (nBurnVer >> 8) & 0xFFF);
  printf("%i games supported.\n\n",nBurnDrvCount);
  printf("Using Pentium Pro: %s\n",USE_PPRO);
  printf("Using MMX: %s\n",USE_MMX);
  printf("Using FinalBurn Alpha code: %s\n",USE_ALPHA);
  printf("Using ComaC modifications: %s\n\n",USE_COMAC);

  ProcesaAllConfigFiles();

  a = 1;
  while (a < ArgN)
  {
    a += ProcesaParam(Args[a], a+1 < ArgN ? Args[a+1] : NULL);
  }

  if (ShowHelp)
  {
    PrintParamsHelp();
    exit(0);
  }

  if (PrintRomsInfo)
  {
    prntRomsInfo(PrintRomsInfo);
    
    BurnLibExit();
    exit(0);
  }
  
  lnxAudioInit();

  nBurnDrvSelect = GetRomNumber(RomName);
  if (Verbose) { printf("Loading \"%s\"...\n",BurnDrvText(1)); }
  
  c = 0;
  while (!BurnDrvGetZipName(&Nombre, c))
  {
    szBzipName[c] = (char *)malloc(strlen(Nombre)+strlen(RomsPath)+1);
    sprintf(szBzipName[c],"%s%s",RomsPath,Nombre);
    c++;
  }

  while (c < BZIP_MAX)
  {
    szBzipName[c++] = NULL;
  }
      
  if (BzipOpen()) { printf("No BzipOpen\n"); exit(1); }
  if (nBzipError) { printf("\n\nBzipError (Open): %i\n",nBzipError); }
  
  if (BurnDrvInit())
  {
    printf("Error loading game.\n");
    exit(1);
  }

  if (nBzipError) { printf("\n\nBzipError: %i\n",nBzipError); }

  if (Verbose >= 2) { printf("%s\n",BzipDetail.szText); }
  if (Verbose >= 2) { printf("%s\n",BzipText.szText); }
  
  BzipClose();
  
  BurnDrvGetScreen(&scrWidth,&scrHeight);
  if (Verbose >= 3) { printf("Width: %i; Height: %i\n",scrWidth,scrHeight); }

  nInputs = 0;
  while (!BurnDrvGetInputInfo(&Ii, nInputs))
  {
    nInputs++;
  }

  Inputs = (struct cmcInput *)malloc(sizeof(struct cmcInput) * nInputs);
  for (a = 0; a < nInputs; a++)
  {
    BurnDrvGetInputInfo(&Ii, a);
    Inputs[a].Type = Ii.nType;
    Inputs[a].pVal = Ii.pVal;
    Inputs[a].Key1 = -1;
    Inputs[a].Key2 = -1;
    Inputs[a].Press1 = 0;
    Inputs[a].Press2 = 0;
    Inputs[a].CenterVal = *(Ii.pVal);

    switch (Ii.nType)
    {
      case 0 :
	Inputs[a].Value1 = 0xFF;
	Inputs[a].Value2 = 0;
	break;
      case 1 :
	Inputs[a].Value1 = 0x7F;
	Inputs[a].Value2 = -0x7F;
//	Inputs[a].CenterVal = 0x80;
	break;
    }

    if (!strcasecmp(Ii.szInfo, "p1 coin")) { Inputs[a].Key1 = '5'; }
    else if (!strcasecmp(Ii.szInfo, "p2 coin")) { Inputs[a].Key1 = '6'; }
    else if (!strcasecmp(Ii.szInfo, "p3 coin")) { Inputs[a].Key1 = '7'; }
    else if (!strcasecmp(Ii.szInfo, "p4 coin")) { Inputs[a].Key1 = '8'; }

    else if (!strcasecmp(Ii.szInfo, "p1 start")) { Inputs[a].Key1 = '1'; }
    else if (!strcasecmp(Ii.szInfo, "p2 start")) { Inputs[a].Key1 = '2'; }
    else if (!strcasecmp(Ii.szInfo, "p3 start")) { Inputs[a].Key1 = '3'; }
    else if (!strcasecmp(Ii.szInfo, "p4 start")) { Inputs[a].Key1 = '4'; }

#ifdef Player2
    else if (!strcasecmp(Ii.szInfo, "p1 up"))
	{ Inputs[a].Key1 = 'i'; Inputs[a].Key2 = 'k'; }
    else if (!strcasecmp(Ii.szInfo, "p1 down"))
	{ Inputs[a].Key1 = 'k'; Inputs[a].Key2 = 'i'; }
    else if (!strcasecmp(Ii.szInfo, "p1 left"))
	{ Inputs[a].Key1 = 'j'; Inputs[a].Key2 = 'l'; }
    else if (!strcasecmp(Ii.szInfo, "p1 right"))
	{ Inputs[a].Key1 = 'l'; Inputs[a].Key2 = 'j'; }

    else if (!strcasecmp(Ii.szInfo, "p2 up"))
	{ Inputs[a].Key1 = SDLK_UP; Inputs[a].Key2 = SDLK_DOWN; }
    else if (!strcasecmp(Ii.szInfo, "p2 down"))
	{ Inputs[a].Key1 = SDLK_DOWN; Inputs[a].Key2 = SDLK_UP; }
    else if (!strcasecmp(Ii.szInfo, "p2 left"))
	{ Inputs[a].Key1 = SDLK_LEFT; Inputs[a].Key2 = SDLK_RIGHT; }
    else if (!strcasecmp(Ii.szInfo, "p2 right"))
	{ Inputs[a].Key1 = SDLK_RIGHT; Inputs[a].Key2 = SDLK_LEFT; }
#else
    else if (!strcasecmp(Ii.szInfo, "p1 up"))
	{ Inputs[a].Key1 = SDLK_UP; Inputs[a].Key2 = SDLK_DOWN; }
    else if (!strcasecmp(Ii.szInfo, "p1 down"))
	{ Inputs[a].Key1 = SDLK_DOWN; Inputs[a].Key2 = SDLK_UP; }
    else if (!strcasecmp(Ii.szInfo, "p1 left"))
	{ Inputs[a].Key1 = SDLK_LEFT; Inputs[a].Key2 = SDLK_RIGHT; }
    else if (!strcasecmp(Ii.szInfo, "p1 right"))
	{ Inputs[a].Key1 = SDLK_RIGHT; Inputs[a].Key2 = SDLK_LEFT; }
#endif

    else if (!strcasecmp(Ii.szInfo, "p1 x-axis"))
       { Inputs[a].Key1 = SDLK_RIGHT; Inputs[a].Key2 = SDLK_LEFT; }
    else if (!strcasecmp(Ii.szInfo, "p1 y-axis"))
       { Inputs[a].Key1 = SDLK_DOWN; Inputs[a].Key2 = SDLK_UP; }

    else if (!strcasecmp(Ii.szInfo, "p1 fire 1")) { Inputs[a].Key1 = 'z'; Inputs[a].CenterVal = 0; }
    else if (!strcasecmp(Ii.szInfo, "p1 fire 2")) { Inputs[a].Key1 = 'x'; }
    else if (!strcasecmp(Ii.szInfo, "p1 fire 3")) { Inputs[a].Key1 = 'c'; Inputs[a].CenterVal = 0; }

    else if (!strcasecmp(Ii.szInfo, "p1 fire 4")) { Inputs[a].Key1 = 'a'; }
    else if (!strcasecmp(Ii.szInfo, "p1 fire 5")) { Inputs[a].Key1 = 's'; }
    else if (!strcasecmp(Ii.szInfo, "p1 fire 6")) { Inputs[a].Key1 = 'd'; }

#ifdef Player2
    else if (!strcasecmp(Ii.szInfo, "p2 fire 1")) { Inputs[a].Key1 = SDLK_KP1; Inputs[a].CenterVal = 0; }
    else if (!strcasecmp(Ii.szInfo, "p2 fire 2")) { Inputs[a].Key1 = SDLK_KP2; }
    else if (!strcasecmp(Ii.szInfo, "p2 fire 3")) { Inputs[a].Key1 = SDLK_KP3; Inputs[a].CenterVal = 0; }

    else if (!strcasecmp(Ii.szInfo, "p2 fire 4")) { Inputs[a].Key1 = SDLK_KP4; }
    else if (!strcasecmp(Ii.szInfo, "p2 fire 5")) { Inputs[a].Key1 = SDLK_KP5; }
    else if (!strcasecmp(Ii.szInfo, "p2 fire 6")) { Inputs[a].Key1 = SDLK_KP6; }
#endif

    else if (!strcasecmp(Ii.szInfo, "reset")) { Inputs[a].Key1 = SDLK_F3; }
    else if (!strcasecmp(Ii.szInfo, "diag")) { Inputs[a].Key1 = SDLK_F2; }
    else if (!strcasecmp(Ii.szInfo, "service")) { Inputs[a].Key1 = '9'; }

    else { Inputs[a].Key1 = -1; }

    *(Inputs[a].pVal) = Inputs[a].CenterVal;

  }

  VideoInit();
  BPPs = InitVideoMode(scrWidth, scrHeight, FullScreen, &BytesPP);

  pBurnDraw = (unsigned char *)malloc(scrWidth*scrHeight*BytesPP);
  DrawBkp = pBurnDraw;
  nBurnPitch = scrWidth*BytesPP;
  nBurnBpp = BytesPP;

  if (Verbose >= 3) { printf("BitsPP: %i; BytesPP: %i\n",BPPs, BytesPP); }


  switch (BPPs)
  {
    default: BurnHighCol = SDL_HighCol; break;
  }

  if (BufferVideo)
  {
    BuffPantalla = (char *)malloc(scrWidth*scrHeight * nBurnBpp);
  }


  TimeFrame = 1000000 / 60;
  CurrTime = useconds();
  InitTime = CurrTime;

  if (Sound) { StartAudioPlay(); }

  while (!Salir)
  {
    pBurnDraw = NULL;

#define WEARELATE ( Sound ? WeAreLate() : CurrTime < (useconds() - TimeFrame) )

#ifndef PROFILE
    while (WEARELATE)
    {
      BurnDrvFrame();

      if (Sound)
      {
#ifdef COMAC
        switch (SFilter)
        {
          case LowPass_Cmc: DspDo(pBurnSoundOut, nBurnSoundLen); break;
          case LowPass2_Cmc: Dsp2Do(pBurnSoundOut, nBurnSoundLen); break;
        }
#endif
        PlayAudio(pBurnSoundOut, nBurnSoundLen);
      }

      CurrTime += TimeFrame;
      LostFrames++;
    }
#endif

    if (BufferVideo)
    {
      pBurnDraw = (unsigned char *)BuffPantalla;
      nBurnPitch = BytesPP * scrWidth;
      BurnDrvFrame();

      pBurnDraw = (unsigned char *)PrePaint(&nBurnPitch);
      for (a = 0; a < scrHeight; a++)
      {
        memmove(pBurnDraw,BuffPantalla, BytesPP*scrWidth);
        BuffPantalla += BytesPP * scrWidth;
        pBurnDraw += nBurnPitch;
      }
      BuffPantalla -= BytesPP * scrWidth * scrHeight;
    }
    else
    {
      pBurnDraw = (unsigned char *)PrePaint(&nBurnPitch);
      BurnDrvFrame();
    }

    Paint(pBurnDraw);

    if (Sound)
    {
#ifdef COMAC
      switch (SFilter)
      {
        case LowPass_Cmc: DspDo(pBurnSoundOut, nBurnSoundLen); break;
        case LowPass2_Cmc: Dsp2Do(pBurnSoundOut, nBurnSoundLen); break;
      }
#endif
      PlayAudio(pBurnSoundOut, nBurnSoundLen);
    }

    CurrTime += TimeFrame;

    while (Tecla = ProcesInput(&Valor))
    {
      if ((Tecla == 27) /*|| (Tecla == 'q')*/)
      {
        Salir = 1;
      }
      else
      {
        for (a = 0; a < nInputs; a++)
        {
          if (Tecla == Inputs[a].Key1)
	  {
            Inputs[a].Press1 = Valor;
	    switch (Inputs[a].Type)
	    {
	      case 0 :
		*(Inputs[a].pVal) = Inputs[a].Press2 ? 0 : Inputs[a].Value1 * Inputs[a].Press1;
		break;
	      case 1 :
		*(Inputs[a].pVal) = Inputs[a].CenterVal +
				    Inputs[a].Press1*Inputs[a].Value1 +
				    Inputs[a].Press2*Inputs[a].Value2;
		break;
	    }
          }
          if (Tecla == Inputs[a].Key2)
	  {
            Inputs[a].Press2 = Valor;
	    switch (Inputs[a].Type)
	    {
	      case 0 :
		*(Inputs[a].pVal) = Inputs[a].Press2 ? 0 : Inputs[a].Value1 * Inputs[a].Press1;
		break;
	      case 1 :
		*(Inputs[a].pVal) = Inputs[a].CenterVal +
				    Inputs[a].Press1*Inputs[a].Value1 +
				    Inputs[a].Press2*Inputs[a].Value2;
		break;
	    }
          }
        }
      }
    }

    if (!Sound)
    {
      while (CurrTime >= useconds())
      {
        usleep(20);
      }
    }
  }

  if (Sound) { AudioExit(); }
  VideoExit();
  BurnLibExit();

  if (Verbose >= 3) { printf("\n\nLost frames: %i\nTime running: %0.3f\n",LostFrames,(CurrTime-InitTime)/1000000.0); }
  return 0;
}
